var GrapesJSSymbols = (function (Backbone) {
  'use strict';

  // Current version.
  var VERSION = '1.13.7';

  // Establish the root object, `window` (`self`) in the browser, `global`
  // on the server, or `this` in some virtual machines. We use `self`
  // instead of `window` for `WebWorker` support.
  var root = (typeof self == 'object' && self.self === self && self) ||
            (typeof global == 'object' && global.global === global && global) ||
            Function('return this')() ||
            {};

  // Save bytes in the minified (but not gzipped) version:
  var ArrayProto = Array.prototype, ObjProto = Object.prototype;
  var SymbolProto = typeof Symbol !== 'undefined' ? Symbol.prototype : null;

  // Create quick reference variables for speed access to core prototypes.
  var push = ArrayProto.push,
      slice = ArrayProto.slice,
      toString = ObjProto.toString,
      hasOwnProperty = ObjProto.hasOwnProperty;

  // Modern feature detection.
  var supportsArrayBuffer = typeof ArrayBuffer !== 'undefined',
      supportsDataView = typeof DataView !== 'undefined';

  // All **ECMAScript 5+** native function implementations that we hope to use
  // are declared here.
  var nativeIsArray = Array.isArray,
      nativeKeys = Object.keys,
      nativeCreate = Object.create,
      nativeIsView = supportsArrayBuffer && ArrayBuffer.isView;

  // Create references to these builtin functions because we override them.
  var _isNaN = isNaN,
      _isFinite = isFinite;

  // Keys in IE < 9 that won't be iterated by `for key in ...` and thus missed.
  var hasEnumBug = !{toString: null}.propertyIsEnumerable('toString');
  var nonEnumerableProps = ['valueOf', 'isPrototypeOf', 'toString',
    'propertyIsEnumerable', 'hasOwnProperty', 'toLocaleString'];

  // The largest integer that can be represented exactly.
  var MAX_ARRAY_INDEX = Math.pow(2, 53) - 1;

  // Some functions take a variable number of arguments, or a few expected
  // arguments at the beginning and then a variable number of values to operate
  // on. This helper accumulates all remaining arguments past the function’s
  // argument length (or an explicit `startIndex`), into an array that becomes
  // the last argument. Similar to ES6’s "rest parameter".
  function restArguments(func, startIndex) {
    startIndex = startIndex == null ? func.length - 1 : +startIndex;
    return function() {
      var length = Math.max(arguments.length - startIndex, 0),
          rest = Array(length),
          index = 0;
      for (; index < length; index++) {
        rest[index] = arguments[index + startIndex];
      }
      switch (startIndex) {
        case 0: return func.call(this, rest);
        case 1: return func.call(this, arguments[0], rest);
        case 2: return func.call(this, arguments[0], arguments[1], rest);
      }
      var args = Array(startIndex + 1);
      for (index = 0; index < startIndex; index++) {
        args[index] = arguments[index];
      }
      args[startIndex] = rest;
      return func.apply(this, args);
    };
  }

  // Is a given variable an object?
  function isObject(obj) {
    var type = typeof obj;
    return type === 'function' || (type === 'object' && !!obj);
  }

  // Is a given value equal to null?
  function isNull(obj) {
    return obj === null;
  }

  // Is a given variable undefined?
  function isUndefined(obj) {
    return obj === void 0;
  }

  // Is a given value a boolean?
  function isBoolean(obj) {
    return obj === true || obj === false || toString.call(obj) === '[object Boolean]';
  }

  // Is a given value a DOM element?
  function isElement(obj) {
    return !!(obj && obj.nodeType === 1);
  }

  // Internal function for creating a `toString`-based type tester.
  function tagTester(name) {
    var tag = '[object ' + name + ']';
    return function(obj) {
      return toString.call(obj) === tag;
    };
  }

  var isString = tagTester('String');

  var isNumber = tagTester('Number');

  var isDate = tagTester('Date');

  var isRegExp = tagTester('RegExp');

  var isError = tagTester('Error');

  var isSymbol = tagTester('Symbol');

  var isArrayBuffer = tagTester('ArrayBuffer');

  var isFunction = tagTester('Function');

  // Optimize `isFunction` if appropriate. Work around some `typeof` bugs in old
  // v8, IE 11 (#1621), Safari 8 (#1929), and PhantomJS (#2236).
  var nodelist = root.document && root.document.childNodes;
  if (typeof /./ != 'function' && typeof Int8Array != 'object' && typeof nodelist != 'function') {
    isFunction = function(obj) {
      return typeof obj == 'function' || false;
    };
  }

  var isFunction$1 = isFunction;

  var hasObjectTag = tagTester('Object');

  // In IE 10 - Edge 13, `DataView` has string tag `'[object Object]'`.
  // In IE 11, the most common among them, this problem also applies to
  // `Map`, `WeakMap` and `Set`.
  // Also, there are cases where an application can override the native
  // `DataView` object, in cases like that we can't use the constructor
  // safely and should just rely on alternate `DataView` checks
  var hasDataViewBug = (
        supportsDataView && (!/\[native code\]/.test(String(DataView)) || hasObjectTag(new DataView(new ArrayBuffer(8))))
      ),
      isIE11 = (typeof Map !== 'undefined' && hasObjectTag(new Map));

  var isDataView = tagTester('DataView');

  // In IE 10 - Edge 13, we need a different heuristic
  // to determine whether an object is a `DataView`.
  // Also, in cases where the native `DataView` is
  // overridden we can't rely on the tag itself.
  function alternateIsDataView(obj) {
    return obj != null && isFunction$1(obj.getInt8) && isArrayBuffer(obj.buffer);
  }

  var isDataView$1 = (hasDataViewBug ? alternateIsDataView : isDataView);

  // Is a given value an array?
  // Delegates to ECMA5's native `Array.isArray`.
  var isArray = nativeIsArray || tagTester('Array');

  // Internal function to check whether `key` is an own property name of `obj`.
  function has$1(obj, key) {
    return obj != null && hasOwnProperty.call(obj, key);
  }

  var isArguments = tagTester('Arguments');

  // Define a fallback version of the method in browsers (ahem, IE < 9), where
  // there isn't any inspectable "Arguments" type.
  (function() {
    if (!isArguments(arguments)) {
      isArguments = function(obj) {
        return has$1(obj, 'callee');
      };
    }
  }());

  var isArguments$1 = isArguments;

  // Is a given object a finite number?
  function isFinite$1(obj) {
    return !isSymbol(obj) && _isFinite(obj) && !isNaN(parseFloat(obj));
  }

  // Is the given value `NaN`?
  function isNaN$1(obj) {
    return isNumber(obj) && _isNaN(obj);
  }

  // Predicate-generating function. Often useful outside of Underscore.
  function constant(value) {
    return function() {
      return value;
    };
  }

  // Common internal logic for `isArrayLike` and `isBufferLike`.
  function createSizePropertyCheck(getSizeProperty) {
    return function(collection) {
      var sizeProperty = getSizeProperty(collection);
      return typeof sizeProperty == 'number' && sizeProperty >= 0 && sizeProperty <= MAX_ARRAY_INDEX;
    }
  }

  // Internal helper to generate a function to obtain property `key` from `obj`.
  function shallowProperty(key) {
    return function(obj) {
      return obj == null ? void 0 : obj[key];
    };
  }

  // Internal helper to obtain the `byteLength` property of an object.
  var getByteLength = shallowProperty('byteLength');

  // Internal helper to determine whether we should spend extensive checks against
  // `ArrayBuffer` et al.
  var isBufferLike = createSizePropertyCheck(getByteLength);

  // Is a given value a typed array?
  var typedArrayPattern = /\[object ((I|Ui)nt(8|16|32)|Float(32|64)|Uint8Clamped|Big(I|Ui)nt64)Array\]/;
  function isTypedArray(obj) {
    // `ArrayBuffer.isView` is the most future-proof, so use it when available.
    // Otherwise, fall back on the above regular expression.
    return nativeIsView ? (nativeIsView(obj) && !isDataView$1(obj)) :
                  isBufferLike(obj) && typedArrayPattern.test(toString.call(obj));
  }

  var isTypedArray$1 = supportsArrayBuffer ? isTypedArray : constant(false);

  // Internal helper to obtain the `length` property of an object.
  var getLength = shallowProperty('length');

  // Internal helper to create a simple lookup structure.
  // `collectNonEnumProps` used to depend on `_.contains`, but this led to
  // circular imports. `emulatedSet` is a one-off solution that only works for
  // arrays of strings.
  function emulatedSet(keys) {
    var hash = {};
    for (var l = keys.length, i = 0; i < l; ++i) hash[keys[i]] = true;
    return {
      contains: function(key) { return hash[key] === true; },
      push: function(key) {
        hash[key] = true;
        return keys.push(key);
      }
    };
  }

  // Internal helper. Checks `keys` for the presence of keys in IE < 9 that won't
  // be iterated by `for key in ...` and thus missed. Extends `keys` in place if
  // needed.
  function collectNonEnumProps(obj, keys) {
    keys = emulatedSet(keys);
    var nonEnumIdx = nonEnumerableProps.length;
    var constructor = obj.constructor;
    var proto = (isFunction$1(constructor) && constructor.prototype) || ObjProto;

    // Constructor is a special case.
    var prop = 'constructor';
    if (has$1(obj, prop) && !keys.contains(prop)) keys.push(prop);

    while (nonEnumIdx--) {
      prop = nonEnumerableProps[nonEnumIdx];
      if (prop in obj && obj[prop] !== proto[prop] && !keys.contains(prop)) {
        keys.push(prop);
      }
    }
  }

  // Retrieve the names of an object's own properties.
  // Delegates to **ECMAScript 5**'s native `Object.keys`.
  function keys(obj) {
    if (!isObject(obj)) return [];
    if (nativeKeys) return nativeKeys(obj);
    var keys = [];
    for (var key in obj) if (has$1(obj, key)) keys.push(key);
    // Ahem, IE < 9.
    if (hasEnumBug) collectNonEnumProps(obj, keys);
    return keys;
  }

  // Is a given array, string, or object empty?
  // An "empty" object has no enumerable own-properties.
  function isEmpty(obj) {
    if (obj == null) return true;
    // Skip the more expensive `toString`-based type checks if `obj` has no
    // `.length`.
    var length = getLength(obj);
    if (typeof length == 'number' && (
      isArray(obj) || isString(obj) || isArguments$1(obj)
    )) return length === 0;
    return getLength(keys(obj)) === 0;
  }

  // Returns whether an object has a given set of `key:value` pairs.
  function isMatch(object, attrs) {
    var _keys = keys(attrs), length = _keys.length;
    if (object == null) return !length;
    var obj = Object(object);
    for (var i = 0; i < length; i++) {
      var key = _keys[i];
      if (attrs[key] !== obj[key] || !(key in obj)) return false;
    }
    return true;
  }

  // If Underscore is called as a function, it returns a wrapped object that can
  // be used OO-style. This wrapper holds altered versions of all functions added
  // through `_.mixin`. Wrapped objects may be chained.
  function _$2(obj) {
    if (obj instanceof _$2) return obj;
    if (!(this instanceof _$2)) return new _$2(obj);
    this._wrapped = obj;
  }

  _$2.VERSION = VERSION;

  // Extracts the result from a wrapped and chained object.
  _$2.prototype.value = function() {
    return this._wrapped;
  };

  // Provide unwrapping proxies for some methods used in engine operations
  // such as arithmetic and JSON stringification.
  _$2.prototype.valueOf = _$2.prototype.toJSON = _$2.prototype.value;

  _$2.prototype.toString = function() {
    return String(this._wrapped);
  };

  // Internal function to wrap or shallow-copy an ArrayBuffer,
  // typed array or DataView to a new view, reusing the buffer.
  function toBufferView(bufferSource) {
    return new Uint8Array(
      bufferSource.buffer || bufferSource,
      bufferSource.byteOffset || 0,
      getByteLength(bufferSource)
    );
  }

  // We use this string twice, so give it a name for minification.
  var tagDataView = '[object DataView]';

  // Internal recursive comparison function for `_.isEqual`.
  function eq(a, b, aStack, bStack) {
    // Identical objects are equal. `0 === -0`, but they aren't identical.
    // See the [Harmony `egal` proposal](https://wiki.ecmascript.org/doku.php?id=harmony:egal).
    if (a === b) return a !== 0 || 1 / a === 1 / b;
    // `null` or `undefined` only equal to itself (strict comparison).
    if (a == null || b == null) return false;
    // `NaN`s are equivalent, but non-reflexive.
    if (a !== a) return b !== b;
    // Exhaust primitive checks
    var type = typeof a;
    if (type !== 'function' && type !== 'object' && typeof b != 'object') return false;
    return deepEq(a, b, aStack, bStack);
  }

  // Internal recursive comparison function for `_.isEqual`.
  function deepEq(a, b, aStack, bStack) {
    // Unwrap any wrapped objects.
    if (a instanceof _$2) a = a._wrapped;
    if (b instanceof _$2) b = b._wrapped;
    // Compare `[[Class]]` names.
    var className = toString.call(a);
    if (className !== toString.call(b)) return false;
    // Work around a bug in IE 10 - Edge 13.
    if (hasDataViewBug && className == '[object Object]' && isDataView$1(a)) {
      if (!isDataView$1(b)) return false;
      className = tagDataView;
    }
    switch (className) {
      // These types are compared by value.
      case '[object RegExp]':
        // RegExps are coerced to strings for comparison (Note: '' + /a/i === '/a/i')
      case '[object String]':
        // Primitives and their corresponding object wrappers are equivalent; thus, `"5"` is
        // equivalent to `new String("5")`.
        return '' + a === '' + b;
      case '[object Number]':
        // `NaN`s are equivalent, but non-reflexive.
        // Object(NaN) is equivalent to NaN.
        if (+a !== +a) return +b !== +b;
        // An `egal` comparison is performed for other numeric values.
        return +a === 0 ? 1 / +a === 1 / b : +a === +b;
      case '[object Date]':
      case '[object Boolean]':
        // Coerce dates and booleans to numeric primitive values. Dates are compared by their
        // millisecond representations. Note that invalid dates with millisecond representations
        // of `NaN` are not equivalent.
        return +a === +b;
      case '[object Symbol]':
        return SymbolProto.valueOf.call(a) === SymbolProto.valueOf.call(b);
      case '[object ArrayBuffer]':
      case tagDataView:
        // Coerce to typed array so we can fall through.
        return deepEq(toBufferView(a), toBufferView(b), aStack, bStack);
    }

    var areArrays = className === '[object Array]';
    if (!areArrays && isTypedArray$1(a)) {
        var byteLength = getByteLength(a);
        if (byteLength !== getByteLength(b)) return false;
        if (a.buffer === b.buffer && a.byteOffset === b.byteOffset) return true;
        areArrays = true;
    }
    if (!areArrays) {
      if (typeof a != 'object' || typeof b != 'object') return false;

      // Objects with different constructors are not equivalent, but `Object`s or `Array`s
      // from different frames are.
      var aCtor = a.constructor, bCtor = b.constructor;
      if (aCtor !== bCtor && !(isFunction$1(aCtor) && aCtor instanceof aCtor &&
                               isFunction$1(bCtor) && bCtor instanceof bCtor)
                          && ('constructor' in a && 'constructor' in b)) {
        return false;
      }
    }
    // Assume equality for cyclic structures. The algorithm for detecting cyclic
    // structures is adapted from ES 5.1 section 15.12.3, abstract operation `JO`.

    // Initializing stack of traversed objects.
    // It's done here since we only need them for objects and arrays comparison.
    aStack = aStack || [];
    bStack = bStack || [];
    var length = aStack.length;
    while (length--) {
      // Linear search. Performance is inversely proportional to the number of
      // unique nested structures.
      if (aStack[length] === a) return bStack[length] === b;
    }

    // Add the first object to the stack of traversed objects.
    aStack.push(a);
    bStack.push(b);

    // Recursively compare objects and arrays.
    if (areArrays) {
      // Compare array lengths to determine if a deep comparison is necessary.
      length = a.length;
      if (length !== b.length) return false;
      // Deep compare the contents, ignoring non-numeric properties.
      while (length--) {
        if (!eq(a[length], b[length], aStack, bStack)) return false;
      }
    } else {
      // Deep compare objects.
      var _keys = keys(a), key;
      length = _keys.length;
      // Ensure that both objects contain the same number of properties before comparing deep equality.
      if (keys(b).length !== length) return false;
      while (length--) {
        // Deep compare each member
        key = _keys[length];
        if (!(has$1(b, key) && eq(a[key], b[key], aStack, bStack))) return false;
      }
    }
    // Remove the first object from the stack of traversed objects.
    aStack.pop();
    bStack.pop();
    return true;
  }

  // Perform a deep comparison to check if two objects are equal.
  function isEqual(a, b) {
    return eq(a, b);
  }

  // Retrieve all the enumerable property names of an object.
  function allKeys(obj) {
    if (!isObject(obj)) return [];
    var keys = [];
    for (var key in obj) keys.push(key);
    // Ahem, IE < 9.
    if (hasEnumBug) collectNonEnumProps(obj, keys);
    return keys;
  }

  // Since the regular `Object.prototype.toString` type tests don't work for
  // some types in IE 11, we use a fingerprinting heuristic instead, based
  // on the methods. It's not great, but it's the best we got.
  // The fingerprint method lists are defined below.
  function ie11fingerprint(methods) {
    var length = getLength(methods);
    return function(obj) {
      if (obj == null) return false;
      // `Map`, `WeakMap` and `Set` have no enumerable keys.
      var keys = allKeys(obj);
      if (getLength(keys)) return false;
      for (var i = 0; i < length; i++) {
        if (!isFunction$1(obj[methods[i]])) return false;
      }
      // If we are testing against `WeakMap`, we need to ensure that
      // `obj` doesn't have a `forEach` method in order to distinguish
      // it from a regular `Map`.
      return methods !== weakMapMethods || !isFunction$1(obj[forEachName]);
    };
  }

  // In the interest of compact minification, we write
  // each string in the fingerprints only once.
  var forEachName = 'forEach',
      hasName = 'has',
      commonInit = ['clear', 'delete'],
      mapTail = ['get', hasName, 'set'];

  // `Map`, `WeakMap` and `Set` each have slightly different
  // combinations of the above sublists.
  var mapMethods = commonInit.concat(forEachName, mapTail),
      weakMapMethods = commonInit.concat(mapTail),
      setMethods = ['add'].concat(commonInit, forEachName, hasName);

  var isMap = isIE11 ? ie11fingerprint(mapMethods) : tagTester('Map');

  var isWeakMap = isIE11 ? ie11fingerprint(weakMapMethods) : tagTester('WeakMap');

  var isSet = isIE11 ? ie11fingerprint(setMethods) : tagTester('Set');

  var isWeakSet = tagTester('WeakSet');

  // Retrieve the values of an object's properties.
  function values(obj) {
    var _keys = keys(obj);
    var length = _keys.length;
    var values = Array(length);
    for (var i = 0; i < length; i++) {
      values[i] = obj[_keys[i]];
    }
    return values;
  }

  // Convert an object into a list of `[key, value]` pairs.
  // The opposite of `_.object` with one argument.
  function pairs(obj) {
    var _keys = keys(obj);
    var length = _keys.length;
    var pairs = Array(length);
    for (var i = 0; i < length; i++) {
      pairs[i] = [_keys[i], obj[_keys[i]]];
    }
    return pairs;
  }

  // Invert the keys and values of an object. The values must be serializable.
  function invert(obj) {
    var result = {};
    var _keys = keys(obj);
    for (var i = 0, length = _keys.length; i < length; i++) {
      result[obj[_keys[i]]] = _keys[i];
    }
    return result;
  }

  // Return a sorted list of the function names available on the object.
  function functions(obj) {
    var names = [];
    for (var key in obj) {
      if (isFunction$1(obj[key])) names.push(key);
    }
    return names.sort();
  }

  // An internal function for creating assigner functions.
  function createAssigner(keysFunc, defaults) {
    return function(obj) {
      var length = arguments.length;
      if (defaults) obj = Object(obj);
      if (length < 2 || obj == null) return obj;
      for (var index = 1; index < length; index++) {
        var source = arguments[index],
            keys = keysFunc(source),
            l = keys.length;
        for (var i = 0; i < l; i++) {
          var key = keys[i];
          if (!defaults || obj[key] === void 0) obj[key] = source[key];
        }
      }
      return obj;
    };
  }

  // Extend a given object with all the properties in passed-in object(s).
  var extend = createAssigner(allKeys);

  // Assigns a given object with all the own properties in the passed-in
  // object(s).
  // (https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object/assign)
  var extendOwn = createAssigner(keys);

  // Fill in a given object with default properties.
  var defaults = createAssigner(allKeys, true);

  // Create a naked function reference for surrogate-prototype-swapping.
  function ctor() {
    return function(){};
  }

  // An internal function for creating a new object that inherits from another.
  function baseCreate(prototype) {
    if (!isObject(prototype)) return {};
    if (nativeCreate) return nativeCreate(prototype);
    var Ctor = ctor();
    Ctor.prototype = prototype;
    var result = new Ctor;
    Ctor.prototype = null;
    return result;
  }

  // Creates an object that inherits from the given prototype object.
  // If additional properties are provided then they will be added to the
  // created object.
  function create(prototype, props) {
    var result = baseCreate(prototype);
    if (props) extendOwn(result, props);
    return result;
  }

  // Create a (shallow-cloned) duplicate of an object.
  function clone(obj) {
    if (!isObject(obj)) return obj;
    return isArray(obj) ? obj.slice() : extend({}, obj);
  }

  // Invokes `interceptor` with the `obj` and then returns `obj`.
  // The primary purpose of this method is to "tap into" a method chain, in
  // order to perform operations on intermediate results within the chain.
  function tap(obj, interceptor) {
    interceptor(obj);
    return obj;
  }

  // Normalize a (deep) property `path` to array.
  // Like `_.iteratee`, this function can be customized.
  function toPath$1(path) {
    return isArray(path) ? path : [path];
  }
  _$2.toPath = toPath$1;

  // Internal wrapper for `_.toPath` to enable minification.
  // Similar to `cb` for `_.iteratee`.
  function toPath(path) {
    return _$2.toPath(path);
  }

  // Internal function to obtain a nested property in `obj` along `path`.
  function deepGet(obj, path) {
    var length = path.length;
    for (var i = 0; i < length; i++) {
      if (obj == null) return void 0;
      obj = obj[path[i]];
    }
    return length ? obj : void 0;
  }

  // Get the value of the (deep) property on `path` from `object`.
  // If any property in `path` does not exist or if the value is
  // `undefined`, return `defaultValue` instead.
  // The `path` is normalized through `_.toPath`.
  function get(object, path, defaultValue) {
    var value = deepGet(object, toPath(path));
    return isUndefined(value) ? defaultValue : value;
  }

  // Shortcut function for checking if an object has a given property directly on
  // itself (in other words, not on a prototype). Unlike the internal `has`
  // function, this public version can also traverse nested properties.
  function has(obj, path) {
    path = toPath(path);
    var length = path.length;
    for (var i = 0; i < length; i++) {
      var key = path[i];
      if (!has$1(obj, key)) return false;
      obj = obj[key];
    }
    return !!length;
  }

  // Keep the identity function around for default iteratees.
  function identity(value) {
    return value;
  }

  // Returns a predicate for checking whether an object has a given set of
  // `key:value` pairs.
  function matcher(attrs) {
    attrs = extendOwn({}, attrs);
    return function(obj) {
      return isMatch(obj, attrs);
    };
  }

  // Creates a function that, when passed an object, will traverse that object’s
  // properties down the given `path`, specified as an array of keys or indices.
  function property(path) {
    path = toPath(path);
    return function(obj) {
      return deepGet(obj, path);
    };
  }

  // Internal function that returns an efficient (for current engines) version
  // of the passed-in callback, to be repeatedly applied in other Underscore
  // functions.
  function optimizeCb(func, context, argCount) {
    if (context === void 0) return func;
    switch (argCount == null ? 3 : argCount) {
      case 1: return function(value) {
        return func.call(context, value);
      };
      // The 2-argument case is omitted because we’re not using it.
      case 3: return function(value, index, collection) {
        return func.call(context, value, index, collection);
      };
      case 4: return function(accumulator, value, index, collection) {
        return func.call(context, accumulator, value, index, collection);
      };
    }
    return function() {
      return func.apply(context, arguments);
    };
  }

  // An internal function to generate callbacks that can be applied to each
  // element in a collection, returning the desired result — either `_.identity`,
  // an arbitrary callback, a property matcher, or a property accessor.
  function baseIteratee(value, context, argCount) {
    if (value == null) return identity;
    if (isFunction$1(value)) return optimizeCb(value, context, argCount);
    if (isObject(value) && !isArray(value)) return matcher(value);
    return property(value);
  }

  // External wrapper for our callback generator. Users may customize
  // `_.iteratee` if they want additional predicate/iteratee shorthand styles.
  // This abstraction hides the internal-only `argCount` argument.
  function iteratee(value, context) {
    return baseIteratee(value, context, Infinity);
  }
  _$2.iteratee = iteratee;

  // The function we call internally to generate a callback. It invokes
  // `_.iteratee` if overridden, otherwise `baseIteratee`.
  function cb(value, context, argCount) {
    if (_$2.iteratee !== iteratee) return _$2.iteratee(value, context);
    return baseIteratee(value, context, argCount);
  }

  // Returns the results of applying the `iteratee` to each element of `obj`.
  // In contrast to `_.map` it returns an object.
  function mapObject(obj, iteratee, context) {
    iteratee = cb(iteratee, context);
    var _keys = keys(obj),
        length = _keys.length,
        results = {};
    for (var index = 0; index < length; index++) {
      var currentKey = _keys[index];
      results[currentKey] = iteratee(obj[currentKey], currentKey, obj);
    }
    return results;
  }

  // Predicate-generating function. Often useful outside of Underscore.
  function noop(){}

  // Generates a function for a given object that returns a given property.
  function propertyOf(obj) {
    if (obj == null) return noop;
    return function(path) {
      return get(obj, path);
    };
  }

  // Run a function **n** times.
  function times(n, iteratee, context) {
    var accum = Array(Math.max(0, n));
    iteratee = optimizeCb(iteratee, context, 1);
    for (var i = 0; i < n; i++) accum[i] = iteratee(i);
    return accum;
  }

  // Return a random integer between `min` and `max` (inclusive).
  function random(min, max) {
    if (max == null) {
      max = min;
      min = 0;
    }
    return min + Math.floor(Math.random() * (max - min + 1));
  }

  // A (possibly faster) way to get the current timestamp as an integer.
  var now = Date.now || function() {
    return new Date().getTime();
  };

  // Internal helper to generate functions for escaping and unescaping strings
  // to/from HTML interpolation.
  function createEscaper(map) {
    var escaper = function(match) {
      return map[match];
    };
    // Regexes for identifying a key that needs to be escaped.
    var source = '(?:' + keys(map).join('|') + ')';
    var testRegexp = RegExp(source);
    var replaceRegexp = RegExp(source, 'g');
    return function(string) {
      string = string == null ? '' : '' + string;
      return testRegexp.test(string) ? string.replace(replaceRegexp, escaper) : string;
    };
  }

  // Internal list of HTML entities for escaping.
  var escapeMap = {
    '&': '&amp;',
    '<': '&lt;',
    '>': '&gt;',
    '"': '&quot;',
    "'": '&#x27;',
    '`': '&#x60;'
  };

  // Function for escaping strings to HTML interpolation.
  var escape = createEscaper(escapeMap);

  // Internal list of HTML entities for unescaping.
  var unescapeMap = invert(escapeMap);

  // Function for unescaping strings from HTML interpolation.
  var unescape = createEscaper(unescapeMap);

  // By default, Underscore uses ERB-style template delimiters. Change the
  // following template settings to use alternative delimiters.
  var templateSettings = _$2.templateSettings = {
    evaluate: /<%([\s\S]+?)%>/g,
    interpolate: /<%=([\s\S]+?)%>/g,
    escape: /<%-([\s\S]+?)%>/g
  };

  // When customizing `_.templateSettings`, if you don't want to define an
  // interpolation, evaluation or escaping regex, we need one that is
  // guaranteed not to match.
  var noMatch = /(.)^/;

  // Certain characters need to be escaped so that they can be put into a
  // string literal.
  var escapes = {
    "'": "'",
    '\\': '\\',
    '\r': 'r',
    '\n': 'n',
    '\u2028': 'u2028',
    '\u2029': 'u2029'
  };

  var escapeRegExp = /\\|'|\r|\n|\u2028|\u2029/g;

  function escapeChar(match) {
    return '\\' + escapes[match];
  }

  // In order to prevent third-party code injection through
  // `_.templateSettings.variable`, we test it against the following regular
  // expression. It is intentionally a bit more liberal than just matching valid
  // identifiers, but still prevents possible loopholes through defaults or
  // destructuring assignment.
  var bareIdentifier = /^\s*(\w|\$)+\s*$/;

  // JavaScript micro-templating, similar to John Resig's implementation.
  // Underscore templating handles arbitrary delimiters, preserves whitespace,
  // and correctly escapes quotes within interpolated code.
  // NB: `oldSettings` only exists for backwards compatibility.
  function template(text, settings, oldSettings) {
    if (!settings && oldSettings) settings = oldSettings;
    settings = defaults({}, settings, _$2.templateSettings);

    // Combine delimiters into one regular expression via alternation.
    var matcher = RegExp([
      (settings.escape || noMatch).source,
      (settings.interpolate || noMatch).source,
      (settings.evaluate || noMatch).source
    ].join('|') + '|$', 'g');

    // Compile the template source, escaping string literals appropriately.
    var index = 0;
    var source = "__p+='";
    text.replace(matcher, function(match, escape, interpolate, evaluate, offset) {
      source += text.slice(index, offset).replace(escapeRegExp, escapeChar);
      index = offset + match.length;

      if (escape) {
        source += "'+\n((__t=(" + escape + "))==null?'':_.escape(__t))+\n'";
      } else if (interpolate) {
        source += "'+\n((__t=(" + interpolate + "))==null?'':__t)+\n'";
      } else if (evaluate) {
        source += "';\n" + evaluate + "\n__p+='";
      }

      // Adobe VMs need the match returned to produce the correct offset.
      return match;
    });
    source += "';\n";

    var argument = settings.variable;
    if (argument) {
      // Insure against third-party code injection. (CVE-2021-23358)
      if (!bareIdentifier.test(argument)) throw new Error(
        'variable is not a bare identifier: ' + argument
      );
    } else {
      // If a variable is not specified, place data values in local scope.
      source = 'with(obj||{}){\n' + source + '}\n';
      argument = 'obj';
    }

    source = "var __t,__p='',__j=Array.prototype.join," +
      "print=function(){__p+=__j.call(arguments,'');};\n" +
      source + 'return __p;\n';

    var render;
    try {
      render = new Function(argument, '_', source);
    } catch (e) {
      e.source = source;
      throw e;
    }

    var template = function(data) {
      return render.call(this, data, _$2);
    };

    // Provide the compiled source as a convenience for precompilation.
    template.source = 'function(' + argument + '){\n' + source + '}';

    return template;
  }

  // Traverses the children of `obj` along `path`. If a child is a function, it
  // is invoked with its parent as context. Returns the value of the final
  // child, or `fallback` if any child is undefined.
  function result(obj, path, fallback) {
    path = toPath(path);
    var length = path.length;
    if (!length) {
      return isFunction$1(fallback) ? fallback.call(obj) : fallback;
    }
    for (var i = 0; i < length; i++) {
      var prop = obj == null ? void 0 : obj[path[i]];
      if (prop === void 0) {
        prop = fallback;
        i = length; // Ensure we don't continue iterating.
      }
      obj = isFunction$1(prop) ? prop.call(obj) : prop;
    }
    return obj;
  }

  // Generate a unique integer id (unique within the entire client session).
  // Useful for temporary DOM ids.
  var idCounter = 0;
  function uniqueId(prefix) {
    var id = ++idCounter + '';
    return prefix ? prefix + id : id;
  }

  // Start chaining a wrapped Underscore object.
  function chain(obj) {
    var instance = _$2(obj);
    instance._chain = true;
    return instance;
  }

  // Internal function to execute `sourceFunc` bound to `context` with optional
  // `args`. Determines whether to execute a function as a constructor or as a
  // normal function.
  function executeBound(sourceFunc, boundFunc, context, callingContext, args) {
    if (!(callingContext instanceof boundFunc)) return sourceFunc.apply(context, args);
    var self = baseCreate(sourceFunc.prototype);
    var result = sourceFunc.apply(self, args);
    if (isObject(result)) return result;
    return self;
  }

  // Partially apply a function by creating a version that has had some of its
  // arguments pre-filled, without changing its dynamic `this` context. `_` acts
  // as a placeholder by default, allowing any combination of arguments to be
  // pre-filled. Set `_.partial.placeholder` for a custom placeholder argument.
  var partial = restArguments(function(func, boundArgs) {
    var placeholder = partial.placeholder;
    var bound = function() {
      var position = 0, length = boundArgs.length;
      var args = Array(length);
      for (var i = 0; i < length; i++) {
        args[i] = boundArgs[i] === placeholder ? arguments[position++] : boundArgs[i];
      }
      while (position < arguments.length) args.push(arguments[position++]);
      return executeBound(func, bound, this, this, args);
    };
    return bound;
  });

  partial.placeholder = _$2;

  // Create a function bound to a given object (assigning `this`, and arguments,
  // optionally).
  var bind = restArguments(function(func, context, args) {
    if (!isFunction$1(func)) throw new TypeError('Bind must be called on a function');
    var bound = restArguments(function(callArgs) {
      return executeBound(func, bound, context, this, args.concat(callArgs));
    });
    return bound;
  });

  // Internal helper for collection methods to determine whether a collection
  // should be iterated as an array or as an object.
  // Related: https://people.mozilla.org/~jorendorff/es6-draft.html#sec-tolength
  // Avoids a very nasty iOS 8 JIT bug on ARM-64. #2094
  var isArrayLike = createSizePropertyCheck(getLength);

  // Internal implementation of a recursive `flatten` function.
  function flatten$1(input, depth, strict, output) {
    output = output || [];
    if (!depth && depth !== 0) {
      depth = Infinity;
    } else if (depth <= 0) {
      return output.concat(input);
    }
    var idx = output.length;
    for (var i = 0, length = getLength(input); i < length; i++) {
      var value = input[i];
      if (isArrayLike(value) && (isArray(value) || isArguments$1(value))) {
        // Flatten current level of array or arguments object.
        if (depth > 1) {
          flatten$1(value, depth - 1, strict, output);
          idx = output.length;
        } else {
          var j = 0, len = value.length;
          while (j < len) output[idx++] = value[j++];
        }
      } else if (!strict) {
        output[idx++] = value;
      }
    }
    return output;
  }

  // Bind a number of an object's methods to that object. Remaining arguments
  // are the method names to be bound. Useful for ensuring that all callbacks
  // defined on an object belong to it.
  var bindAll = restArguments(function(obj, keys) {
    keys = flatten$1(keys, false, false);
    var index = keys.length;
    if (index < 1) throw new Error('bindAll must be passed function names');
    while (index--) {
      var key = keys[index];
      obj[key] = bind(obj[key], obj);
    }
    return obj;
  });

  // Memoize an expensive function by storing its results.
  function memoize(func, hasher) {
    var memoize = function(key) {
      var cache = memoize.cache;
      var address = '' + (hasher ? hasher.apply(this, arguments) : key);
      if (!has$1(cache, address)) cache[address] = func.apply(this, arguments);
      return cache[address];
    };
    memoize.cache = {};
    return memoize;
  }

  // Delays a function for the given number of milliseconds, and then calls
  // it with the arguments supplied.
  var delay = restArguments(function(func, wait, args) {
    return setTimeout(function() {
      return func.apply(null, args);
    }, wait);
  });

  // Defers a function, scheduling it to run after the current call stack has
  // cleared.
  var defer = partial(delay, _$2, 1);

  // Returns a function, that, when invoked, will only be triggered at most once
  // during a given window of time. Normally, the throttled function will run
  // as much as it can, without ever going more than once per `wait` duration;
  // but if you'd like to disable the execution on the leading edge, pass
  // `{leading: false}`. To disable execution on the trailing edge, ditto.
  function throttle(func, wait, options) {
    var timeout, context, args, result;
    var previous = 0;
    if (!options) options = {};

    var later = function() {
      previous = options.leading === false ? 0 : now();
      timeout = null;
      result = func.apply(context, args);
      if (!timeout) context = args = null;
    };

    var throttled = function() {
      var _now = now();
      if (!previous && options.leading === false) previous = _now;
      var remaining = wait - (_now - previous);
      context = this;
      args = arguments;
      if (remaining <= 0 || remaining > wait) {
        if (timeout) {
          clearTimeout(timeout);
          timeout = null;
        }
        previous = _now;
        result = func.apply(context, args);
        if (!timeout) context = args = null;
      } else if (!timeout && options.trailing !== false) {
        timeout = setTimeout(later, remaining);
      }
      return result;
    };

    throttled.cancel = function() {
      clearTimeout(timeout);
      previous = 0;
      timeout = context = args = null;
    };

    return throttled;
  }

  // When a sequence of calls of the returned function ends, the argument
  // function is triggered. The end of a sequence is defined by the `wait`
  // parameter. If `immediate` is passed, the argument function will be
  // triggered at the beginning of the sequence instead of at the end.
  function debounce(func, wait, immediate) {
    var timeout, previous, args, result, context;

    var later = function() {
      var passed = now() - previous;
      if (wait > passed) {
        timeout = setTimeout(later, wait - passed);
      } else {
        timeout = null;
        if (!immediate) result = func.apply(context, args);
        // This check is needed because `func` can recursively invoke `debounced`.
        if (!timeout) args = context = null;
      }
    };

    var debounced = restArguments(function(_args) {
      context = this;
      args = _args;
      previous = now();
      if (!timeout) {
        timeout = setTimeout(later, wait);
        if (immediate) result = func.apply(context, args);
      }
      return result;
    });

    debounced.cancel = function() {
      clearTimeout(timeout);
      timeout = args = context = null;
    };

    return debounced;
  }

  // Returns the first function passed as an argument to the second,
  // allowing you to adjust arguments, run code before and after, and
  // conditionally execute the original function.
  function wrap(func, wrapper) {
    return partial(wrapper, func);
  }

  // Returns a negated version of the passed-in predicate.
  function negate(predicate) {
    return function() {
      return !predicate.apply(this, arguments);
    };
  }

  // Returns a function that is the composition of a list of functions, each
  // consuming the return value of the function that follows.
  function compose() {
    var args = arguments;
    var start = args.length - 1;
    return function() {
      var i = start;
      var result = args[start].apply(this, arguments);
      while (i--) result = args[i].call(this, result);
      return result;
    };
  }

  // Returns a function that will only be executed on and after the Nth call.
  function after(times, func) {
    return function() {
      if (--times < 1) {
        return func.apply(this, arguments);
      }
    };
  }

  // Returns a function that will only be executed up to (but not including) the
  // Nth call.
  function before(times, func) {
    var memo;
    return function() {
      if (--times > 0) {
        memo = func.apply(this, arguments);
      }
      if (times <= 1) func = null;
      return memo;
    };
  }

  // Returns a function that will be executed at most one time, no matter how
  // often you call it. Useful for lazy initialization.
  var once = partial(before, 2);

  // Returns the first key on an object that passes a truth test.
  function findKey(obj, predicate, context) {
    predicate = cb(predicate, context);
    var _keys = keys(obj), key;
    for (var i = 0, length = _keys.length; i < length; i++) {
      key = _keys[i];
      if (predicate(obj[key], key, obj)) return key;
    }
  }

  // Internal function to generate `_.findIndex` and `_.findLastIndex`.
  function createPredicateIndexFinder(dir) {
    return function(array, predicate, context) {
      predicate = cb(predicate, context);
      var length = getLength(array);
      var index = dir > 0 ? 0 : length - 1;
      for (; index >= 0 && index < length; index += dir) {
        if (predicate(array[index], index, array)) return index;
      }
      return -1;
    };
  }

  // Returns the first index on an array-like that passes a truth test.
  var findIndex = createPredicateIndexFinder(1);

  // Returns the last index on an array-like that passes a truth test.
  var findLastIndex = createPredicateIndexFinder(-1);

  // Use a comparator function to figure out the smallest index at which
  // an object should be inserted so as to maintain order. Uses binary search.
  function sortedIndex(array, obj, iteratee, context) {
    iteratee = cb(iteratee, context, 1);
    var value = iteratee(obj);
    var low = 0, high = getLength(array);
    while (low < high) {
      var mid = Math.floor((low + high) / 2);
      if (iteratee(array[mid]) < value) low = mid + 1; else high = mid;
    }
    return low;
  }

  // Internal function to generate the `_.indexOf` and `_.lastIndexOf` functions.
  function createIndexFinder(dir, predicateFind, sortedIndex) {
    return function(array, item, idx) {
      var i = 0, length = getLength(array);
      if (typeof idx == 'number') {
        if (dir > 0) {
          i = idx >= 0 ? idx : Math.max(idx + length, i);
        } else {
          length = idx >= 0 ? Math.min(idx + 1, length) : idx + length + 1;
        }
      } else if (sortedIndex && idx && length) {
        idx = sortedIndex(array, item);
        return array[idx] === item ? idx : -1;
      }
      if (item !== item) {
        idx = predicateFind(slice.call(array, i, length), isNaN$1);
        return idx >= 0 ? idx + i : -1;
      }
      for (idx = dir > 0 ? i : length - 1; idx >= 0 && idx < length; idx += dir) {
        if (array[idx] === item) return idx;
      }
      return -1;
    };
  }

  // Return the position of the first occurrence of an item in an array,
  // or -1 if the item is not included in the array.
  // If the array is large and already in sort order, pass `true`
  // for **isSorted** to use binary search.
  var indexOf = createIndexFinder(1, findIndex, sortedIndex);

  // Return the position of the last occurrence of an item in an array,
  // or -1 if the item is not included in the array.
  var lastIndexOf = createIndexFinder(-1, findLastIndex);

  // Return the first value which passes a truth test.
  function find$1(obj, predicate, context) {
    var keyFinder = isArrayLike(obj) ? findIndex : findKey;
    var key = keyFinder(obj, predicate, context);
    if (key !== void 0 && key !== -1) return obj[key];
  }

  // Convenience version of a common use case of `_.find`: getting the first
  // object containing specific `key:value` pairs.
  function findWhere(obj, attrs) {
    return find$1(obj, matcher(attrs));
  }

  // The cornerstone for collection functions, an `each`
  // implementation, aka `forEach`.
  // Handles raw objects in addition to array-likes. Treats all
  // sparse array-likes as if they were dense.
  function each(obj, iteratee, context) {
    iteratee = optimizeCb(iteratee, context);
    var i, length;
    if (isArrayLike(obj)) {
      for (i = 0, length = obj.length; i < length; i++) {
        iteratee(obj[i], i, obj);
      }
    } else {
      var _keys = keys(obj);
      for (i = 0, length = _keys.length; i < length; i++) {
        iteratee(obj[_keys[i]], _keys[i], obj);
      }
    }
    return obj;
  }

  // Return the results of applying the iteratee to each element.
  function map(obj, iteratee, context) {
    iteratee = cb(iteratee, context);
    var _keys = !isArrayLike(obj) && keys(obj),
        length = (_keys || obj).length,
        results = Array(length);
    for (var index = 0; index < length; index++) {
      var currentKey = _keys ? _keys[index] : index;
      results[index] = iteratee(obj[currentKey], currentKey, obj);
    }
    return results;
  }

  // Internal helper to create a reducing function, iterating left or right.
  function createReduce(dir) {
    // Wrap code that reassigns argument variables in a separate function than
    // the one that accesses `arguments.length` to avoid a perf hit. (#1991)
    var reducer = function(obj, iteratee, memo, initial) {
      var _keys = !isArrayLike(obj) && keys(obj),
          length = (_keys || obj).length,
          index = dir > 0 ? 0 : length - 1;
      if (!initial) {
        memo = obj[_keys ? _keys[index] : index];
        index += dir;
      }
      for (; index >= 0 && index < length; index += dir) {
        var currentKey = _keys ? _keys[index] : index;
        memo = iteratee(memo, obj[currentKey], currentKey, obj);
      }
      return memo;
    };

    return function(obj, iteratee, memo, context) {
      var initial = arguments.length >= 3;
      return reducer(obj, optimizeCb(iteratee, context, 4), memo, initial);
    };
  }

  // **Reduce** builds up a single result from a list of values, aka `inject`,
  // or `foldl`.
  var reduce = createReduce(1);

  // The right-associative version of reduce, also known as `foldr`.
  var reduceRight = createReduce(-1);

  // Return all the elements that pass a truth test.
  function filter(obj, predicate, context) {
    var results = [];
    predicate = cb(predicate, context);
    each(obj, function(value, index, list) {
      if (predicate(value, index, list)) results.push(value);
    });
    return results;
  }

  // Return all the elements for which a truth test fails.
  function reject(obj, predicate, context) {
    return filter(obj, negate(cb(predicate)), context);
  }

  // Determine whether all of the elements pass a truth test.
  function every(obj, predicate, context) {
    predicate = cb(predicate, context);
    var _keys = !isArrayLike(obj) && keys(obj),
        length = (_keys || obj).length;
    for (var index = 0; index < length; index++) {
      var currentKey = _keys ? _keys[index] : index;
      if (!predicate(obj[currentKey], currentKey, obj)) return false;
    }
    return true;
  }

  // Determine if at least one element in the object passes a truth test.
  function some(obj, predicate, context) {
    predicate = cb(predicate, context);
    var _keys = !isArrayLike(obj) && keys(obj),
        length = (_keys || obj).length;
    for (var index = 0; index < length; index++) {
      var currentKey = _keys ? _keys[index] : index;
      if (predicate(obj[currentKey], currentKey, obj)) return true;
    }
    return false;
  }

  // Determine if the array or object contains a given item (using `===`).
  function contains(obj, item, fromIndex, guard) {
    if (!isArrayLike(obj)) obj = values(obj);
    if (typeof fromIndex != 'number' || guard) fromIndex = 0;
    return indexOf(obj, item, fromIndex) >= 0;
  }

  // Invoke a method (with arguments) on every item in a collection.
  var invoke = restArguments(function(obj, path, args) {
    var contextPath, func;
    if (isFunction$1(path)) {
      func = path;
    } else {
      path = toPath(path);
      contextPath = path.slice(0, -1);
      path = path[path.length - 1];
    }
    return map(obj, function(context) {
      var method = func;
      if (!method) {
        if (contextPath && contextPath.length) {
          context = deepGet(context, contextPath);
        }
        if (context == null) return void 0;
        method = context[path];
      }
      return method == null ? method : method.apply(context, args);
    });
  });

  // Convenience version of a common use case of `_.map`: fetching a property.
  function pluck(obj, key) {
    return map(obj, property(key));
  }

  // Convenience version of a common use case of `_.filter`: selecting only
  // objects containing specific `key:value` pairs.
  function where(obj, attrs) {
    return filter(obj, matcher(attrs));
  }

  // Return the maximum element (or element-based computation).
  function max(obj, iteratee, context) {
    var result = -Infinity, lastComputed = -Infinity,
        value, computed;
    if (iteratee == null || (typeof iteratee == 'number' && typeof obj[0] != 'object' && obj != null)) {
      obj = isArrayLike(obj) ? obj : values(obj);
      for (var i = 0, length = obj.length; i < length; i++) {
        value = obj[i];
        if (value != null && value > result) {
          result = value;
        }
      }
    } else {
      iteratee = cb(iteratee, context);
      each(obj, function(v, index, list) {
        computed = iteratee(v, index, list);
        if (computed > lastComputed || (computed === -Infinity && result === -Infinity)) {
          result = v;
          lastComputed = computed;
        }
      });
    }
    return result;
  }

  // Return the minimum element (or element-based computation).
  function min(obj, iteratee, context) {
    var result = Infinity, lastComputed = Infinity,
        value, computed;
    if (iteratee == null || (typeof iteratee == 'number' && typeof obj[0] != 'object' && obj != null)) {
      obj = isArrayLike(obj) ? obj : values(obj);
      for (var i = 0, length = obj.length; i < length; i++) {
        value = obj[i];
        if (value != null && value < result) {
          result = value;
        }
      }
    } else {
      iteratee = cb(iteratee, context);
      each(obj, function(v, index, list) {
        computed = iteratee(v, index, list);
        if (computed < lastComputed || (computed === Infinity && result === Infinity)) {
          result = v;
          lastComputed = computed;
        }
      });
    }
    return result;
  }

  // Safely create a real, live array from anything iterable.
  var reStrSymbol = /[^\ud800-\udfff]|[\ud800-\udbff][\udc00-\udfff]|[\ud800-\udfff]/g;
  function toArray(obj) {
    if (!obj) return [];
    if (isArray(obj)) return slice.call(obj);
    if (isString(obj)) {
      // Keep surrogate pair characters together.
      return obj.match(reStrSymbol);
    }
    if (isArrayLike(obj)) return map(obj, identity);
    return values(obj);
  }

  // Sample **n** random values from a collection using the modern version of the
  // [Fisher-Yates shuffle](https://en.wikipedia.org/wiki/Fisher–Yates_shuffle).
  // If **n** is not specified, returns a single random element.
  // The internal `guard` argument allows it to work with `_.map`.
  function sample(obj, n, guard) {
    if (n == null || guard) {
      if (!isArrayLike(obj)) obj = values(obj);
      return obj[random(obj.length - 1)];
    }
    var sample = toArray(obj);
    var length = getLength(sample);
    n = Math.max(Math.min(n, length), 0);
    var last = length - 1;
    for (var index = 0; index < n; index++) {
      var rand = random(index, last);
      var temp = sample[index];
      sample[index] = sample[rand];
      sample[rand] = temp;
    }
    return sample.slice(0, n);
  }

  // Shuffle a collection.
  function shuffle(obj) {
    return sample(obj, Infinity);
  }

  // Sort the object's values by a criterion produced by an iteratee.
  function sortBy(obj, iteratee, context) {
    var index = 0;
    iteratee = cb(iteratee, context);
    return pluck(map(obj, function(value, key, list) {
      return {
        value: value,
        index: index++,
        criteria: iteratee(value, key, list)
      };
    }).sort(function(left, right) {
      var a = left.criteria;
      var b = right.criteria;
      if (a !== b) {
        if (a > b || a === void 0) return 1;
        if (a < b || b === void 0) return -1;
      }
      return left.index - right.index;
    }), 'value');
  }

  // An internal function used for aggregate "group by" operations.
  function group(behavior, partition) {
    return function(obj, iteratee, context) {
      var result = partition ? [[], []] : {};
      iteratee = cb(iteratee, context);
      each(obj, function(value, index) {
        var key = iteratee(value, index, obj);
        behavior(result, value, key);
      });
      return result;
    };
  }

  // Groups the object's values by a criterion. Pass either a string attribute
  // to group by, or a function that returns the criterion.
  var groupBy = group(function(result, value, key) {
    if (has$1(result, key)) result[key].push(value); else result[key] = [value];
  });

  // Indexes the object's values by a criterion, similar to `_.groupBy`, but for
  // when you know that your index values will be unique.
  var indexBy = group(function(result, value, key) {
    result[key] = value;
  });

  // Counts instances of an object that group by a certain criterion. Pass
  // either a string attribute to count by, or a function that returns the
  // criterion.
  var countBy = group(function(result, value, key) {
    if (has$1(result, key)) result[key]++; else result[key] = 1;
  });

  // Split a collection into two arrays: one whose elements all pass the given
  // truth test, and one whose elements all do not pass the truth test.
  var partition = group(function(result, value, pass) {
    result[pass ? 0 : 1].push(value);
  }, true);

  // Return the number of elements in a collection.
  function size(obj) {
    if (obj == null) return 0;
    return isArrayLike(obj) ? obj.length : keys(obj).length;
  }

  // Internal `_.pick` helper function to determine whether `key` is an enumerable
  // property name of `obj`.
  function keyInObj(value, key, obj) {
    return key in obj;
  }

  // Return a copy of the object only containing the allowed properties.
  var pick = restArguments(function(obj, keys) {
    var result = {}, iteratee = keys[0];
    if (obj == null) return result;
    if (isFunction$1(iteratee)) {
      if (keys.length > 1) iteratee = optimizeCb(iteratee, keys[1]);
      keys = allKeys(obj);
    } else {
      iteratee = keyInObj;
      keys = flatten$1(keys, false, false);
      obj = Object(obj);
    }
    for (var i = 0, length = keys.length; i < length; i++) {
      var key = keys[i];
      var value = obj[key];
      if (iteratee(value, key, obj)) result[key] = value;
    }
    return result;
  });

  // Return a copy of the object without the disallowed properties.
  var omit = restArguments(function(obj, keys) {
    var iteratee = keys[0], context;
    if (isFunction$1(iteratee)) {
      iteratee = negate(iteratee);
      if (keys.length > 1) context = keys[1];
    } else {
      keys = map(flatten$1(keys, false, false), String);
      iteratee = function(value, key) {
        return !contains(keys, key);
      };
    }
    return pick(obj, iteratee, context);
  });

  // Returns everything but the last entry of the array. Especially useful on
  // the arguments object. Passing **n** will return all the values in
  // the array, excluding the last N.
  function initial(array, n, guard) {
    return slice.call(array, 0, Math.max(0, array.length - (n == null || guard ? 1 : n)));
  }

  // Get the first element of an array. Passing **n** will return the first N
  // values in the array. The **guard** check allows it to work with `_.map`.
  function first(array, n, guard) {
    if (array == null || array.length < 1) return n == null || guard ? void 0 : [];
    if (n == null || guard) return array[0];
    return initial(array, array.length - n);
  }

  // Returns everything but the first entry of the `array`. Especially useful on
  // the `arguments` object. Passing an **n** will return the rest N values in the
  // `array`.
  function rest(array, n, guard) {
    return slice.call(array, n == null || guard ? 1 : n);
  }

  // Get the last element of an array. Passing **n** will return the last N
  // values in the array.
  function last(array, n, guard) {
    if (array == null || array.length < 1) return n == null || guard ? void 0 : [];
    if (n == null || guard) return array[array.length - 1];
    return rest(array, Math.max(0, array.length - n));
  }

  // Trim out all falsy values from an array.
  function compact(array) {
    return filter(array, Boolean);
  }

  // Flatten out an array, either recursively (by default), or up to `depth`.
  // Passing `true` or `false` as `depth` means `1` or `Infinity`, respectively.
  function flatten(array, depth) {
    return flatten$1(array, depth, false);
  }

  // Take the difference between one array and a number of other arrays.
  // Only the elements present in just the first array will remain.
  var difference = restArguments(function(array, rest) {
    rest = flatten$1(rest, true, true);
    return filter(array, function(value){
      return !contains(rest, value);
    });
  });

  // Return a version of the array that does not contain the specified value(s).
  var without = restArguments(function(array, otherArrays) {
    return difference(array, otherArrays);
  });

  // Produce a duplicate-free version of the array. If the array has already
  // been sorted, you have the option of using a faster algorithm.
  // The faster algorithm will not work with an iteratee if the iteratee
  // is not a one-to-one function, so providing an iteratee will disable
  // the faster algorithm.
  function uniq(array, isSorted, iteratee, context) {
    if (!isBoolean(isSorted)) {
      context = iteratee;
      iteratee = isSorted;
      isSorted = false;
    }
    if (iteratee != null) iteratee = cb(iteratee, context);
    var result = [];
    var seen = [];
    for (var i = 0, length = getLength(array); i < length; i++) {
      var value = array[i],
          computed = iteratee ? iteratee(value, i, array) : value;
      if (isSorted && !iteratee) {
        if (!i || seen !== computed) result.push(value);
        seen = computed;
      } else if (iteratee) {
        if (!contains(seen, computed)) {
          seen.push(computed);
          result.push(value);
        }
      } else if (!contains(result, value)) {
        result.push(value);
      }
    }
    return result;
  }

  // Produce an array that contains the union: each distinct element from all of
  // the passed-in arrays.
  var union = restArguments(function(arrays) {
    return uniq(flatten$1(arrays, true, true));
  });

  // Produce an array that contains every item shared between all the
  // passed-in arrays.
  function intersection(array) {
    var result = [];
    var argsLength = arguments.length;
    for (var i = 0, length = getLength(array); i < length; i++) {
      var item = array[i];
      if (contains(result, item)) continue;
      var j;
      for (j = 1; j < argsLength; j++) {
        if (!contains(arguments[j], item)) break;
      }
      if (j === argsLength) result.push(item);
    }
    return result;
  }

  // Complement of zip. Unzip accepts an array of arrays and groups
  // each array's elements on shared indices.
  function unzip(array) {
    var length = (array && max(array, getLength).length) || 0;
    var result = Array(length);

    for (var index = 0; index < length; index++) {
      result[index] = pluck(array, index);
    }
    return result;
  }

  // Zip together multiple lists into a single array -- elements that share
  // an index go together.
  var zip = restArguments(unzip);

  // Converts lists into objects. Pass either a single array of `[key, value]`
  // pairs, or two parallel arrays of the same length -- one of keys, and one of
  // the corresponding values. Passing by pairs is the reverse of `_.pairs`.
  function object(list, values) {
    var result = {};
    for (var i = 0, length = getLength(list); i < length; i++) {
      if (values) {
        result[list[i]] = values[i];
      } else {
        result[list[i][0]] = list[i][1];
      }
    }
    return result;
  }

  // Generate an integer Array containing an arithmetic progression. A port of
  // the native Python `range()` function. See
  // [the Python documentation](https://docs.python.org/library/functions.html#range).
  function range(start, stop, step) {
    if (stop == null) {
      stop = start || 0;
      start = 0;
    }
    if (!step) {
      step = stop < start ? -1 : 1;
    }

    var length = Math.max(Math.ceil((stop - start) / step), 0);
    var range = Array(length);

    for (var idx = 0; idx < length; idx++, start += step) {
      range[idx] = start;
    }

    return range;
  }

  // Chunk a single array into multiple arrays, each containing `count` or fewer
  // items.
  function chunk(array, count) {
    if (count == null || count < 1) return [];
    var result = [];
    var i = 0, length = array.length;
    while (i < length) {
      result.push(slice.call(array, i, i += count));
    }
    return result;
  }

  // Helper function to continue chaining intermediate results.
  function chainResult(instance, obj) {
    return instance._chain ? _$2(obj).chain() : obj;
  }

  // Add your own custom functions to the Underscore object.
  function mixin(obj) {
    each(functions(obj), function(name) {
      var func = _$2[name] = obj[name];
      _$2.prototype[name] = function() {
        var args = [this._wrapped];
        push.apply(args, arguments);
        return chainResult(this, func.apply(_$2, args));
      };
    });
    return _$2;
  }

  // Add all mutator `Array` functions to the wrapper.
  each(['pop', 'push', 'reverse', 'shift', 'sort', 'splice', 'unshift'], function(name) {
    var method = ArrayProto[name];
    _$2.prototype[name] = function() {
      var obj = this._wrapped;
      if (obj != null) {
        method.apply(obj, arguments);
        if ((name === 'shift' || name === 'splice') && obj.length === 0) {
          delete obj[0];
        }
      }
      return chainResult(this, obj);
    };
  });

  // Add all accessor `Array` functions to the wrapper.
  each(['concat', 'join', 'slice'], function(name) {
    var method = ArrayProto[name];
    _$2.prototype[name] = function() {
      var obj = this._wrapped;
      if (obj != null) obj = method.apply(obj, arguments);
      return chainResult(this, obj);
    };
  });

  // Named Exports
  // =============

  var allExports = /*#__PURE__*/Object.freeze({
    __proto__: null,
    VERSION: VERSION,
    after: after,
    all: every,
    allKeys: allKeys,
    any: some,
    assign: extendOwn,
    before: before,
    bind: bind,
    bindAll: bindAll,
    chain: chain,
    chunk: chunk,
    clone: clone,
    collect: map,
    compact: compact,
    compose: compose,
    constant: constant,
    contains: contains,
    countBy: countBy,
    create: create,
    debounce: debounce,
    default: _$2,
    defaults: defaults,
    defer: defer,
    delay: delay,
    detect: find$1,
    difference: difference,
    drop: rest,
    each: each,
    escape: escape,
    every: every,
    extend: extend,
    extendOwn: extendOwn,
    filter: filter,
    find: find$1,
    findIndex: findIndex,
    findKey: findKey,
    findLastIndex: findLastIndex,
    findWhere: findWhere,
    first: first,
    flatten: flatten,
    foldl: reduce,
    foldr: reduceRight,
    forEach: each,
    functions: functions,
    get: get,
    groupBy: groupBy,
    has: has,
    head: first,
    identity: identity,
    include: contains,
    includes: contains,
    indexBy: indexBy,
    indexOf: indexOf,
    initial: initial,
    inject: reduce,
    intersection: intersection,
    invert: invert,
    invoke: invoke,
    isArguments: isArguments$1,
    isArray: isArray,
    isArrayBuffer: isArrayBuffer,
    isBoolean: isBoolean,
    isDataView: isDataView$1,
    isDate: isDate,
    isElement: isElement,
    isEmpty: isEmpty,
    isEqual: isEqual,
    isError: isError,
    isFinite: isFinite$1,
    isFunction: isFunction$1,
    isMap: isMap,
    isMatch: isMatch,
    isNaN: isNaN$1,
    isNull: isNull,
    isNumber: isNumber,
    isObject: isObject,
    isRegExp: isRegExp,
    isSet: isSet,
    isString: isString,
    isSymbol: isSymbol,
    isTypedArray: isTypedArray$1,
    isUndefined: isUndefined,
    isWeakMap: isWeakMap,
    isWeakSet: isWeakSet,
    iteratee: iteratee,
    keys: keys,
    last: last,
    lastIndexOf: lastIndexOf,
    map: map,
    mapObject: mapObject,
    matcher: matcher,
    matches: matcher,
    max: max,
    memoize: memoize,
    methods: functions,
    min: min,
    mixin: mixin,
    negate: negate,
    noop: noop,
    now: now,
    object: object,
    omit: omit,
    once: once,
    pairs: pairs,
    partial: partial,
    partition: partition,
    pick: pick,
    pluck: pluck,
    property: property,
    propertyOf: propertyOf,
    random: random,
    range: range,
    reduce: reduce,
    reduceRight: reduceRight,
    reject: reject,
    rest: rest,
    restArguments: restArguments,
    result: result,
    sample: sample,
    select: filter,
    shuffle: shuffle,
    size: size,
    some: some,
    sortBy: sortBy,
    sortedIndex: sortedIndex,
    tail: rest,
    take: first,
    tap: tap,
    template: template,
    templateSettings: templateSettings,
    throttle: throttle,
    times: times,
    toArray: toArray,
    toPath: toPath$1,
    transpose: unzip,
    unescape: unescape,
    union: union,
    uniq: uniq,
    unique: uniq,
    uniqueId: uniqueId,
    unzip: unzip,
    values: values,
    where: where,
    without: without,
    wrap: wrap,
    zip: zip
  });

  // Default Export
  // ==============
  // In this module, we mix our bundled exports into the `_` object and export
  // the result. This is analogous to setting `module.exports = _` in CommonJS.
  // Hence, this module is also the entry point of our UMD bundle and the package
  // entry point for CommonJS and AMD users. In other words, this is (the source
  // of) the module you are interfacing with when you do any of the following:
  //
  // ```js
  // // CommonJS
  // var _ = require('underscore');
  //
  // // AMD
  // define(['underscore'], function(_) {...});
  //
  // // UMD in the browser
  // // _ is available as a global variable
  // ```

  // Add all of the Underscore functions to the wrapper object.
  var _$1 = mixin(allExports);
  // Legacy Node.js API.
  _$1._ = _$1;

  // type SymbolAttributes = {
  //   id: string,
  //   model: Component,
  //   label?: string,
  //   icon?: string,
  //   instances?: Map<string, Component>,
  // }
  const SYMBOL_ID_ATTRIBUTE = 'symbolId';
  const SYMBOL_CHILD_ID_ATTRIBUTE = 'symbolChildId';
  const SYMBOL_SYNC_ATTRIBUTE = 'symbolSync';
  /**
   * A Symbol class holds the data about a symbol: label, icon
   * The `model` attribute is a grapesjs Component used to create new instances
   * The `instances` attribute is a Map of grapesjs Components kept in sync with the model
   * The model is kept up to date by calling the apply* methods
   *
   * @member {string} attributes.label
   * @member {string} attributes.icon
   * @member {Component} attributes.model
   * @member {Map.<string, Component>} attributes.instances
   *
   * @class
   */
  let Symbol$1 = class Symbol extends Backbone.Model {
      /**
       * Default options passed to the constructor
       */
      //defaults: {
      //  label: 'New Symbol',
      //  icon: 'fa-question',
      //},
      /**
       * @param {{ label: ?string, icon: ?string }} attributes
       * @param {Object} model - to be converted to Component and stored in attributes.model
       * Notes:
       * - `attributes.instances` will initially be empty until addInstance is called by the Symbols class (onAdd method)
       * - `attributes.model` may initially be a Component (creation of a Symbol) or JSON data (loaded symbol from storage). It is always converted to a Component in `initialize`
       *
       */
      initialize() {
          // Check required attributes
          if (!this.has('model'))
              throw new Error('Could not create Symbol: model is required');
          // Init the required instances on the symbol
          if (!this.has('instances')) {
              this.set('instances', new Map());
          }
          if (this.collection) { // This is false during unit tests
              // Get a ref to grapesjs editor
              const editor = this.collection.editor;
              // `attributes.model` may initially be a Component (creation of a Symbol) or JSON data (loaded symbol from storage). It is always converted to a Component in `initialize`
              // in which case we convert model to a real component
              // TODO: Needs review
              const model = this.get('model');
              if (!model.cid) { // FIXME: should be typeof model = 'string'
                  const [modelComp] = editor.addComponents([model]);
                  this.set('model', modelComp);
              }
              // Make sure the symbol instances are undoable
              editor.UndoManager.add(this);
          }
      }
      /**
       * Return a shallow copy of the model's attributes for JSON
       * stringification.
       * @return {Object}
       * @private
       */
      toJSON(opts = {}) {
          const obj = Backbone.Model.prototype.toJSON.call(this, opts);
          delete obj.instances;
          return obj;
      }
      /**
       * Get all instances as an Array, except the `excludeOne` one
       * @param callback which receives the instances
       * @param excludeOne - optionally exclude one component
       * @param addOne - optionally add one component, typically pass the symbol's `model` attribute when needed
       * @returns The symbol instances
       * @private
       */
      getAll(addOne = null, excludeOne = null) {
          const values = Array.from(this.get('instances').values());
          return (addOne ? [addOne] : []).concat(excludeOne ? values.filter(inst => inst.cid != excludeOne.cid) : values);
      }
      /**
       * Browse all instances and their children matching the changed component
       * Includes the `model` of this symbol
       * Will not include the provided instance `srcInst` nor `srcChild` as they are the ones which changed
       * @param srcInst - the instance of this symbol containing `child`
       * @param srcChild - the child which has the changes
       */
      browseInstancesAndModel(srcInst, srcChildren, cbk) {
          this.getAll(this.get('model'), srcInst)
              .forEach(dstInst => {
              const dstChildren = srcChildren
                  .map(srcChild => {
                  // Get a child or the root
                  return srcChild.has(SYMBOL_ID_ATTRIBUTE)
                      ? dstInst // this is the root
                      : find(dstInst, srcChild.get(SYMBOL_CHILD_ID_ATTRIBUTE)); // this is a child
              });
              cbk(dstChildren, dstInst);
          });
      }
      /**
       * Apply css classes to all instances and their children according to changes of a component
       * Also update the `model` attribute of this symbol
       * @param srcInst - the instance of this symbol containing `child`
       * @param srcChild - the child which has the changes
       */
      applyClasses(srcInst, srcChild) {
          if (srcInst.get(SYMBOL_SYNC_ATTRIBUTE) === false)
              return;
          this.browseInstancesAndModel(srcInst, [srcChild], ([dstChild], dstInst) => {
              if (dstInst.get(SYMBOL_SYNC_ATTRIBUTE) === false)
                  return;
              if (dstChild) {
                  dstChild.setClass(srcChild.getClasses());
              }
              else {
                  console.error(`Could not sync classes for symbol ${this.cid}: ${srcChild.get(SYMBOL_CHILD_ID_ATTRIBUTE)} not found in ${dstInst.cid}`);
              }
          });
      }
      getIndex(parent, symbolChildId) {
          // TODO: Needs review
          return parent.components().toArray()
              .findIndex(c => c.get(SYMBOL_CHILD_ID_ATTRIBUTE) === symbolChildId);
      }
      /**
       * Update attributes of all instances and their children according to changes of a component
       * Also update the `model` attribute of this symbol
       * @param srcInst - the instance of this symbol containing `child`
       * @param parent - the element whose children have changed
       * @param srcChild - the child which has the changes
       */
      applyChildren(srcInst, parent, srcChild) {
          if (srcInst.get(SYMBOL_SYNC_ATTRIBUTE) === false)
              return;
          if (!parent)
              throw new Error(`Could not sync children for symbol ${this.cid}: ${srcChild.cid} has no parent`);
          // Get all instances of this symbol
          const allInst = all(srcInst)
              .filter(inst => inst.get(SYMBOL_SYNC_ATTRIBUTE) !== false);
          // Handle the create/update/remove cases
          if (allInst.includes(srcChild)) {
              // The child is in the instance
              const symbolChildId = srcChild.get(SYMBOL_CHILD_ID_ATTRIBUTE);
              // Case of a child being duplicated inside the symbol
              const isDuplicate = !!symbolChildId && allInst
                  .filter(c => c.get(SYMBOL_CHILD_ID_ATTRIBUTE) === symbolChildId && c.parent() === parent).length > 1;
              if (symbolChildId && !isDuplicate) {
                  // Case of a moving child inside the instance
                  this.browseInstancesAndModel(srcInst, [srcChild, parent], ([dstChild, dstParent], dstInst) => {
                      if (dstChild && dstParent) {
                          dstParent.append(dstChild, { at: srcChild.index() });
                      }
                      else {
                          console.error(`Could not sync child for symbol ${this.cid}: ${srcChild.get(SYMBOL_CHILD_ID_ATTRIBUTE)} not found in ${dstInst.cid}`, { dstChild, dstParent });
                      }
                  });
              }
              else {
                  // this is a new child
                  all(srcChild)
                      // Force new symbolChildId for `srcChild` and its children (excluding symbols)
                      //   because `c` might be a duplicate of another child
                      //   this happens when we duplicate a component inside a symbol
                      .forEach(c => initSymbolChild(c, true));
                  this.browseInstancesAndModel(srcInst, [parent], ([dstParent], dstInst) => {
                      if (dstParent) {
                          const clone = srcChild.clone();
                          dstParent.append(clone, { at: srcChild.index() });
                      }
                      else {
                          console.error(`Could not sync attributes for symbol ${this.cid}: ${srcChild.get(SYMBOL_CHILD_ID_ATTRIBUTE)} not found in ${dstInst.cid}`);
                      }
                  });
              }
          }
          else {
              // Child is not there anymore
              // Beware: srcChild has no parent nor view anymore
              this.browseInstancesAndModel(srcInst, [srcChild], ([dstChild], dstInst) => {
                  if (dstChild) {
                      dstChild.remove();
                  }
                  else {
                      console.error(`Could not sync attributes for symbol ${this.cid}: ${srcChild.get(SYMBOL_CHILD_ID_ATTRIBUTE)} not found in ${dstInst.cid}`);
                  }
              });
              // this child is not part of a symbol anymore
              srcChild.set(SYMBOL_CHILD_ID_ATTRIBUTE);
          }
      }
      /**
       * Update attributes of all instances and their children according to changes of a component
       * Also update the `model` attribute of this symbol
       * @param srcInst - the instance of this symbol containing `child`
       * @param srcChild - the child which has the changes
       */
      applyAttributes(srcInst, srcChild) {
          if (srcInst.get(SYMBOL_SYNC_ATTRIBUTE) === false)
              return;
          this.browseInstancesAndModel(srcInst, [srcChild], ([dstChild], dstInst) => {
              if (dstInst.get(SYMBOL_SYNC_ATTRIBUTE) === false)
                  return;
              if (dstChild) {
                  // doesnt work: dstChild.setAttributes(srcChild.getAttributes())
                  dstChild.attributes = srcChild.attributes;
              }
              else {
                  console.error(`Could not sync attributes for symbol ${this.cid}: ${srcChild.get(SYMBOL_CHILD_ID_ATTRIBUTE)} not found in ${dstInst.cid}`);
              }
          });
      }
      /**
       * Update text content of all instances and their children according to changes of a component
       * Also update the `model` attribute of this symbol
       * Will not update the provided instance `inst` as it is the one which changed
       * @param srcInst - the instance of this symbol containing `child`
       * @param srcChild - the child which has the changes
       */
      applyContent(srcInst, srcChild) {
          if (srcInst.get(SYMBOL_SYNC_ATTRIBUTE) === false)
              return;
          // Store the caret position in the contenteditable container
          const el = srcChild.getCurrentView().el;
          const caret = getCaret(el);
          this.browseInstancesAndModel(srcInst, [srcChild], ([dstChild], dstInst) => {
              if (dstInst.get(SYMBOL_SYNC_ATTRIBUTE) === false)
                  return;
              if (dstChild) {
                  if (dstChild.get('type') === 'text') { // FIXME: sometimes type is ""
                      // Sets the new content
                      dstChild.components(srcChild.getCurrentView().el.innerHTML);
                  }
                  else {
                      console.error('applyContent, NOT A TEXT', dstChild, dstChild.get('type'));
                  }
              }
              else {
                  console.error(`Could not sync content for symbol ${this.cid}: ${srcChild.get(SYMBOL_CHILD_ID_ATTRIBUTE)} not found in ${dstInst.cid}`);
              }
          });
          // Restore the caret position in the contenteditable container
          // TODO: need review
          // FIXME: Why is the caret reset after we change the components which do not have the focus?
          setTimeout(() => {
              // After dom update
              setCaret(el, caret);
          });
      }
      /**
       * Update styles of all instances and their children according to changes of a component
       * Also update the `model` attribute of this symbol
       * Will not update the provided instance `inst` as it is the one which changed
       * @param srcInst - the instance of this symbol containing `child`
       * @param srcChild - the child which has the changes
       * @param changed - the changed styles
       * @param removed - the removed styles
       */
      applyStyle(srcInst, srcChild, changed, removed) {
          if (srcInst.get(SYMBOL_SYNC_ATTRIBUTE) === false)
              return;
          this.browseInstancesAndModel(srcInst, [srcChild], ([dstChild], dstInst) => {
              if (dstInst.get(SYMBOL_SYNC_ATTRIBUTE) === false)
                  return;
              if (dstChild) {
                  dstChild.setStyle({
                      ...dstChild.getStyle(),
                      ...changed,
                  });
                  removed.forEach(styleName => dstChild.removeStyle(styleName));
              }
              else {
                  console.error(`Could not sync content for symbol ${this.cid}: ${srcChild.get(SYMBOL_CHILD_ID_ATTRIBUTE)} not found in ${dstInst.cid}`);
              }
          });
      }
      /**
       * Add a component to this symbol `instances`
       * Called at init when editor triggers a 'components:add' event
       * @param c
       */
      addInstance(c) {
          this.get('instances').set(c.cid, c); // here we use cid as `instances` is built and not saved
      }
      /**
       * @return {Component} The created instance, ready to be added to a component
       */
      createInstance() {
          // Clone the model
          const inst = this.get('model').clone();
          // Add the component to the symbol instances
          this.addInstance(inst);
          // Let the caller add it to a component
          return inst;
      }
      /**
       * @param c - a component
       * @return {Boolean} true if the component is a symbol
       */
      isInstance(c) {
          return !!c.get(SYMBOL_ID_ATTRIBUTE)
              && this.get('instances').has(c.cid);
      }
      /**
       * unlink all instances of a symbol
       */
      unlinkAll() {
          this.get('instances').forEach((c) => this.unlink(c));
      }
      /**
       * unlink an instance from a symbol
       * - remove the symbol ID from the attributes
       * - remove `symbolChildId` from all the children until they are symbols
       * - remove the reference in instances
       */
      unlink(c) {
          c.set(SYMBOL_ID_ATTRIBUTE);
          this.get('instances').delete(c.cid);
          children(c)
              .forEach(child => child.set(SYMBOL_CHILD_ID_ATTRIBUTE));
      }
  };
  /**
   * @param c - a component, supposedly an instance of a symbol
   * @return the symbol ID if the component is a symbol
   */
  function getSymbolId(c) {
      return c.get(SYMBOL_ID_ATTRIBUTE);
  }
  /**
   * Init a component to be this symbol's `model`
   * Also init the component's children
   * @param {Component} c
   */
  function initModel(c, { icon, label, symbolId }) {
      // check that it is not part of a Symbol already
      if (c.has(SYMBOL_ID_ATTRIBUTE)) {
          throw new Error('Could not init Symbol model: the model has already been init');
      }
      // This is the symbol cid
      c.set(SYMBOL_ID_ATTRIBUTE, symbolId);
      // add symbol data
      c.set('icon', `<span class="fa ${icon}"></span>`);
      // Show that this is a symbol, add an icon to the toolbar UI
      const toolbar = c.get('toolbar');
      // FIXME: somehow this happens twice => we should not have to do this check
      if (!toolbar.find(t => !!t.isSymbol)) {
          toolbar.push({
              attributes: {
                  class: 'fa fa-ban on fa-diamond',
                  title: label,
              },
              command: 'do:nothing',
              isSymbol: true, // prevent add 2 buttons
          });
      }
      // init children
      children(c)
          .forEach(child => initSymbolChild(child));
  }
  /**
   * Init a component to be this symbol's `model`'s child
   * @param {Component} c
   */
  function initSymbolChild(c, force = false) {
      if (force || !c.has(SYMBOL_CHILD_ID_ATTRIBUTE)) {
          c.set(SYMBOL_CHILD_ID_ATTRIBUTE, c.cid);
      }
  }
  /**
   * create a new symbol ou of a component
   * the component and its children will be init
   * the component will be cloned and stored as the model
   * @return {Symbol}
   */
  function createSymbol(editor, c, attributes) {
      var _a;
      const symbolId = (_a = attributes.symbolId) !== null && _a !== void 0 ? _a : `s_${uniqueId()}_${new Date().getTime()}`;
      // If the component is in a symbol, we need to update all instances
      const inst = closestInstance(c);
      // Init component with symbolId and children
      initModel(c, {
          ...attributes,
          symbolId,
      });
      // Create a Symbol
      const s = new Symbol$1({
          ...attributes,
          id: symbolId,
          // Clone the component, store a model
          model: c.clone(),
      });
      // Store a ref
      s.addInstance(c);
      // Handle the case where the new symbol is a child of another symbol
      if (inst) {
          // For all instances containing c, make c an instance of the new symbolId
          const parentSymbolId = getSymbolId(inst);
          const parentSymbol = editor.Symbols.get(parentSymbolId);
          // For each child of the new symbol
          all(c)
              // For each instance of the parent symbol (containing a soon to be instance of s)
              .forEach(child => {
              // Here child is a component of the new symbol
              parentSymbol.getAll(null, inst)
                  .forEach(otherInst => {
                  var _a;
                  // For each instance of s and its children
                  const otherChild = find(otherInst, child.get(SYMBOL_CHILD_ID_ATTRIBUTE));
                  console.log('otherChild', (_a = otherChild === null || otherChild === void 0 ? void 0 : otherChild.view) === null || _a === void 0 ? void 0 : _a.el, otherChild === null || otherChild === void 0 ? void 0 : otherChild.get(SYMBOL_ID_ATTRIBUTE), otherChild === null || otherChild === void 0 ? void 0 : otherChild.get(SYMBOL_CHILD_ID_ATTRIBUTE));
                  otherChild === null || otherChild === void 0 ? void 0 : otherChild.set(SYMBOL_ID_ATTRIBUTE, symbolId);
                  otherChild === null || otherChild === void 0 ? void 0 : otherChild.set(SYMBOL_CHILD_ID_ATTRIBUTE, child.get(SYMBOL_CHILD_ID_ATTRIBUTE));
                  // Add the new instance to the symbol
                  if (child === c) {
                      if (otherChild) {
                          s.addInstance(otherChild);
                      }
                      else {
                          console.error('Could not find child', child.get(SYMBOL_CHILD_ID_ATTRIBUTE));
                      }
                  }
              });
          });
      }
      return s;
  }

  /**
   * set editor as dirty
   */
  function setDirty(editor) {
      try {
          const curr = editor.getDirtyCount() || 0;
          editor.getModel().set('changesCount', curr + 1);
      }
      catch (e) {
          // this occures in headless mode and UT
      }
  }
  /**
   * browse all pages and retrieve all website components
   */
  function getAllComponentsFromEditor(editor) {
      const res = [];
      editor.Pages.getAll()
          .forEach(page => {
          page.getMainComponent()
              .onAll(c => res.push(c));
      });
      return res;
  }
  /**
   * Get all the children excluding symbols children
   * @param {Component} c - the root component
   * @returns {(Component|null)} the root component's children
   */
  function children(c) {
      const children = c.components().toArray();
      return children
          .flatMap(child => all(child));
  }
  /**
   * Get an array of the component + its children excluding symbols children
   * @param {Component} c - the root component
   * @returns {(Component|null)} the root component itself and its children
   */
  function all(c) {
      const children = c.components().toArray();
      return [c]
          .concat(children
          .flatMap(child => {
          if (hasSymbolId(child))
              return [child];
          return all(child);
      }));
  }
  /**
   * Find a component in a component's children, with a given symbolChildId or symbolId
   * @param {Component} c - the root component
   * @param {string} cid - the ID we are looking for
   * @returns {(Component|null)} the component itself or one of its children
   */
  function find(c, symbolChildId) {
      if (c.get('symbolChildId') === symbolChildId) {
          return c;
      }
      else {
          // check the children components
          let found = null;
          c.components()
              // Does not work properly, why? .find(comp => find(comp, symbolChildId))
              .forEach(comp => {
              if (!found)
                  found = find(comp, symbolChildId);
          });
          return found;
      }
  }
  /**
   * find the first symbol in the parents (or the element itself)
   * @private
   */
  function closestInstance(c) {
      let ptr = c;
      while (ptr && !hasSymbolId(ptr)) {
          ptr = ptr.parent();
      }
      return ptr;
  }
  /**
   * @param {Component} c - a component
   * @return {Boolean} true if the component has a symbol id
   */
  function hasSymbolId(c) {
      return !!c.get('symbolId');
  }
  async function wait(ms = 0) {
      return new Promise(resolve => setTimeout(resolve, ms));
  }
  /**
   * Get an array of the indexes of the node in its parent nodes
   * @example <div><div></div><div><div id="test"/> => returns [1, 0] for #test
   */
  function getNodePath(root, node) {
      const path = [];
      let pointer = node;
      while (pointer && pointer !== root) {
          const parent = pointer.parentNode;
          const children = Array.from(parent.childNodes);
          const nodeIndex = children.indexOf(pointer); // any because it should be a ChildNode but eslint does not know ChildNode
          path.push(nodeIndex);
          pointer = parent;
      }
      return path;
  }
  /**
   * Get an array of the indexes of the node in its parent nodes
   * @example <div><div></div><div><div id="test"/> => returns [1, 0] for #test
   */
  function getNode(root, path) {
      const mutablePath = [...path];
      let result = root;
      while (result && mutablePath.length) {
          result = result.childNodes[mutablePath.pop()];
      }
      return result;
  }
  /**
   * Gets the caret position
   */
  function getCaret(el) {
      var _a, _b;
      const win = el.ownerDocument.defaultView;
      const sel = win.getSelection();
      const range = sel.rangeCount ? sel.getRangeAt(0) : null;
      const pos = (_a = range === null || range === void 0 ? void 0 : range.startOffset) !== null && _a !== void 0 ? _a : 0;
      const caretEl = (_b = range === null || range === void 0 ? void 0 : range.commonAncestorContainer) !== null && _b !== void 0 ? _b : el;
      const path = getNodePath(el, caretEl);
      return { path, pos };
  }
  /**
   * Sets the caret position
   */
  function setCaret(el, { path, pos }) {
      const textNode = getNode(el, path);
      if (textNode) {
          const win = el.ownerDocument.defaultView;
          const sel = win.getSelection();
          sel.removeAllRanges();
          const range = document.createRange();
          range.selectNodeContents(textNode);
          range.collapse(false);
          range.setStart(textNode, pos);
          range.setEnd(textNode, pos);
          sel.addRange(range);
      }
      else {
          console.error('Could not keep the caret position', { el, path });
      }
  }
  /**
   * find the all the symbols in the parents (or the element itself)
   * @private
   */
  function allParentInstances(c, includeSelf) {
      const result = [];
      let ptr = c ;
      while (ptr) {
          if (hasSymbolId(ptr))
              result.push(ptr);
          ptr = ptr.parent();
      }
      return result;
  }
  /**
   * find the all the symbols in the children (or the element itself)
   */
  function allChildrenInstances(c, includeSelf) {
      const children = c.components().toArray();
      if (includeSelf && hasSymbolId(c))
          ;
      return [c]
          .concat(children
          .flatMap(child => allChildrenInstances(child, true)) // include self only for the subsequent levels
      );
  }
  /**
   * Find if a parent is also a child of the symbol
   */
  function allowDrop({ target, parent }) {
      const allParents = allParentInstances(parent);
      const allChildren = allChildrenInstances(target, false);
      return !allParents.find(p => allChildren.find(c => getSymbolId(c) === getSymbolId(p)));
  }

  var Storage = (editor) => {
      editor.on('storage:start:store', (objectToStore) => {
          objectToStore.symbols = editor.Symbols.toJSON();
      });
      editor.on('storage:end:load', (resultObject) => {
          if (resultObject.symbols) {
              editor.Symbols.reset();
              editor.Symbols.set(resultObject.symbols);
              // update sybols with existing components
              // call setSymbols on load because
              // the `storage:end:load` event is fired after the components are loaded
              editor.Symbols.updateComponents(getAllComponentsFromEditor(editor));
          }
      });
  };

  class Symbols extends Backbone.Collection {
      constructor(models, { editor, options, ...opts }) {
          super(models, opts);
          this.updating = false;
          this.editor = editor;
          this.options = options;
          if (!options.headless) {
              this.initEvents();
          }
          // Make sure the symbol CRUD operations are undoable
          this.editor.UndoManager.add(this);
      }
      disableUndo(disable) {
          if (disable)
              this.editor.UndoManager.stop();
          else
              this.editor.UndoManager.start();
      }
      async preventUndo(cbk) {
          this.editor.UndoManager.stop();
          await cbk();
          this.editor.UndoManager.start();
      }
      initEvents() {
          this.editor.on('component:create', c => this.onAdd(c));
          this.editor.on('component:update:components', (parent, comp) => this.onUpdateChildren(parent, comp));
          this.editor.on('component:update:attributes', c => this.onUpdateAttributes(c));
          this.editor.on('component:update:classes', c => this.onUpdateClasses(c));
          this.editor.on('component:input', c => this.onUpdateContent(c));
          this.editor.on('styleable:change', cssRule => {
              // Sometimes the event is fired with a Component instead of a CssRule
              if (!cssRule.getComponent)
                  return;
              if (!cssRule.changed)
                  return;
              this.onStyleChanged(cssRule);
          });
          this.editor.on('component:drag', ({ target, parent }) => this.onDrag({ target, parent }));
          //this.editor.on('undo', () => {
          //  this.updating = true
          //  setTimeout(() => {
          //    this.updating = false
          //  }, 1000)
          //})
      }
      /**
       * Update sybols with existing components
       * This is used on load only
       * TODO: Use `storage:end:load`? But this event is fired after the components are loaded
       * TODO: Needs review
       * @private
       */
      updateComponents(components) {
          components.forEach(c => this.onAdd(c));
      }
      /**
       * Prevent drop on a symbol into itself or things similar
       */
      onDrag({ target, parent }) {
          if ((parent === null || parent === void 0 ? void 0 : parent.get('droppable')) && !allowDrop({ target, parent })) {
              // Prevent drop
              parent.set('droppable', false);
              // Reset after drop
              this.editor.once('component:drag:end', () => {
                  parent.set('droppable', true);
              });
          }
      }
      /**
       * Add a component to a symbol
       * This is useful only when loading new HTML content
       * When loading a new component which is a symbol,
       *   add a ref to the component in its symbol.get('instances')
       * Export this method for unit tests
       * TODO: Needs review
       * @private
       */
      onAdd(c) {
          const symbolId = getSymbolId(c);
          if (symbolId) {
              const symbol = this.get(symbolId);
              if (symbol) {
                  if (symbol.isInstance(c)) {
                      console.warn('Could not add instance', c, `It is already an instance of symbol with id ${symbolId}`);
                  }
                  else {
                      // This is probably a duplication of a symbol
                      // Or we have just loaded the website
                      // Make it an instance
                      symbol.addInstance(c);
                  }
              }
              else {
                  // The symbol is not yet loaded
                  setTimeout(() => {
                      // Check again
                      if (this.get(symbolId)) ;
                      else {
                          console.error(`Could not make component with id \`${c.getId()}\` an instance of symbol with id \`${symbolId}\`: symbol not found`);
                          this.editor.runCommand('notifications:add', {
                              type: 'error',
                              group: 'Symbols errors',
                              message: `There is a problem with this component: it is supposed to be an instance of a symbol, but the symbol is not found. Symbol id: ${symbolId}`,
                              componentId: c.getId(),
                          });
                      }
                  });
              }
          }
      }
      /**
       * A component's components() has changed
       */
      async onUpdateChildren(parent, component) {
          if (this.updating)
              return;
          const inst = closestInstance(parent);
          if (inst) {
              const symbolId = getSymbolId(inst);
              const s = this.get(symbolId);
              if (s) {
                  // wait for the component's children to be changed
                  // I couldn't find an event like `component:update:components:after`
                  // TODO: need review
                  await wait();
                  this.updating = true;
                  await this.preventUndo(async () => {
                      s.applyChildren(inst, parent, component);
                  });
                  this.updating = false;
              }
              else {
                  console.warn('Could not update instance', component, ': could not find the symbol with id', symbolId);
              }
          }
      }
      /**
       * A component's attributes has changed
       */
      onUpdateAttributes(c) {
          if (this.updating)
              return;
          const inst = closestInstance(c);
          if (inst) {
              const symbolId = getSymbolId(inst);
              const s = this.get(symbolId);
              if (s) {
                  this.updating = true;
                  s.applyAttributes(inst, c);
                  this.updating = false;
              }
              else {
                  console.warn('Could not update instance', c, ': could not find the symbol with id', symbolId);
              }
          }
      }
      /**
       * A component's css classes have changed
       */
      async onUpdateClasses(c) {
          if (this.updating)
              return;
          const inst = closestInstance(c);
          if (inst) {
              const symbolId = getSymbolId(inst);
              const s = this.get(symbolId);
              if (s) {
                  await wait(); // Needed for undo to work
                  this.updating = true;
                  this.preventUndo(() => {
                      s.applyClasses(inst, c);
                  });
                  this.updating = false;
              }
              else {
                  console.warn('Could not update instance', c, ': could not find the symbol with id', symbolId);
              }
          }
      }
      /**
       * A component's text content has changed
       */
      onUpdateContent(c) {
          if (this.updating)
              return;
          const inst = closestInstance(c);
          if (inst) {
              const symbolId = getSymbolId(inst);
              const s = this.get(symbolId);
              if (s) {
                  this.updating = true;
                  this.preventUndo(() => {
                      s.applyContent(inst, c);
                  });
                  this.updating = false;
              }
              else {
                  console.warn('Could not update instance', c, ': could not find the symbol with id', symbolId);
              }
          }
      }
      /**
       * A component's style has changed
       * TODO: Needs review: isn't the style supposed to be just an attribute => we should not need to sync it, just attributes?
       */
      onStyleChanged(cssRule) {
          if (this.updating)
              return;
          const c = cssRule.getComponent();
          const { style } = cssRule.changed;
          if (c && style) {
              const inst = closestInstance(c);
              if (inst) {
                  const symbolId = getSymbolId(inst);
                  const s = this.get(symbolId);
                  if (s) {
                      // Keep only changed values
                      // TODO: Needs review - isn't cssRule.changed supposed to be only what changed?
                      const changed = Object.entries(style)
                          .filter(([key, value]) => cssRule.previousAttributes().style[key] !== value || !cssRule.previousAttributes().style[key])
                          .reduce((result, [key, value]) => {
                          return {
                              ...result,
                              [key]: value,
                          };
                      }, {});
                      // Removed keys
                      const removed = Object.keys(cssRule.previousAttributes().style)
                          .filter(key => !style[key]);
                      if (Object.values(changed).length > 0 || removed.length > 0) {
                          this.updating = true;
                          s.applyStyle(inst, c, changed, removed);
                          this.updating = false;
                      }
                  }
                  else {
                      console.warn('Could not update instance', c, ': could not find the symbol with id', symbolId);
                  }
              }
          }
      }
  }
  // From https://stackoverflow.com/questions/19673089/how-to-properly-define-backbone-collection-using-typescript
  Symbols.prototype.model = Symbol$1;

  /**
   * @license
   * Copyright 2017 Google LLC
   * SPDX-License-Identifier: BSD-3-Clause
   */
  const t$1=globalThis,i$1=t$1.trustedTypes,s=i$1?i$1.createPolicy("lit-html",{createHTML:t=>t}):void 0,e$2="$lit$",h=`lit$${Math.random().toFixed(9).slice(2)}$`,o$1="?"+h,n=`<${o$1}>`,r=document,l=()=>r.createComment(""),c=t=>null===t||"object"!=typeof t&&"function"!=typeof t,a=Array.isArray,u=t=>a(t)||"function"==typeof t?.[Symbol.iterator],d="[ \t\n\f\r]",f=/<(?:(!--|\/[^a-zA-Z])|(\/?[a-zA-Z][^>\s]*)|(\/?$))/g,v=/-->/g,_=/>/g,m=RegExp(`>|${d}(?:([^\\s"'>=/]+)(${d}*=${d}*(?:[^ \t\n\f\r"'\`<>=]|("|')|))|$)`,"g"),p=/'/g,g=/"/g,$=/^(?:script|style|textarea|title)$/i,y=t=>(i,...s)=>({_$litType$:t,strings:i,values:s}),x=y(1),T=Symbol.for("lit-noChange"),E=Symbol.for("lit-nothing"),A=new WeakMap,C=r.createTreeWalker(r,129);function P(t,i){if(!a(t)||!t.hasOwnProperty("raw"))throw Error("invalid template strings array");return void 0!==s?s.createHTML(i):i}const V=(t,i)=>{const s=t.length-1,o=[];let r,l=2===i?"<svg>":3===i?"<math>":"",c=f;for(let i=0;i<s;i++){const s=t[i];let a,u,d=-1,y=0;for(;y<s.length&&(c.lastIndex=y,u=c.exec(s),null!==u);)y=c.lastIndex,c===f?"!--"===u[1]?c=v:void 0!==u[1]?c=_:void 0!==u[2]?($.test(u[2])&&(r=RegExp("</"+u[2],"g")),c=m):void 0!==u[3]&&(c=m):c===m?">"===u[0]?(c=r??f,d=-1):void 0===u[1]?d=-2:(d=c.lastIndex-u[2].length,a=u[1],c=void 0===u[3]?m:'"'===u[3]?g:p):c===g||c===p?c=m:c===v||c===_?c=f:(c=m,r=void 0);const x=c===m&&t[i+1].startsWith("/>")?" ":"";l+=c===f?s+n:d>=0?(o.push(a),s.slice(0,d)+e$2+s.slice(d)+h+x):s+h+(-2===d?i:x);}return [P(t,l+(t[s]||"<?>")+(2===i?"</svg>":3===i?"</math>":"")),o]};class N{constructor({strings:t,_$litType$:s},n){let r;this.parts=[];let c=0,a=0;const u=t.length-1,d=this.parts,[f,v]=V(t,s);if(this.el=N.createElement(f,n),C.currentNode=this.el.content,2===s||3===s){const t=this.el.content.firstChild;t.replaceWith(...t.childNodes);}for(;null!==(r=C.nextNode())&&d.length<u;){if(1===r.nodeType){if(r.hasAttributes())for(const t of r.getAttributeNames())if(t.endsWith(e$2)){const i=v[a++],s=r.getAttribute(t).split(h),e=/([.?@])?(.*)/.exec(i);d.push({type:1,index:c,name:e[2],strings:s,ctor:"."===e[1]?H:"?"===e[1]?I:"@"===e[1]?L:k}),r.removeAttribute(t);}else t.startsWith(h)&&(d.push({type:6,index:c}),r.removeAttribute(t));if($.test(r.tagName)){const t=r.textContent.split(h),s=t.length-1;if(s>0){r.textContent=i$1?i$1.emptyScript:"";for(let i=0;i<s;i++)r.append(t[i],l()),C.nextNode(),d.push({type:2,index:++c});r.append(t[s],l());}}}else if(8===r.nodeType)if(r.data===o$1)d.push({type:2,index:c});else {let t=-1;for(;-1!==(t=r.data.indexOf(h,t+1));)d.push({type:7,index:c}),t+=h.length-1;}c++;}}static createElement(t,i){const s=r.createElement("template");return s.innerHTML=t,s}}function S(t,i,s=t,e){if(i===T)return i;let h=void 0!==e?s._$Co?.[e]:s._$Cl;const o=c(i)?void 0:i._$litDirective$;return h?.constructor!==o&&(h?._$AO?.(false),void 0===o?h=void 0:(h=new o(t),h._$AT(t,s,e)),void 0!==e?(s._$Co??=[])[e]=h:s._$Cl=h),void 0!==h&&(i=S(t,h._$AS(t,i.values),h,e)),i}class M{constructor(t,i){this._$AV=[],this._$AN=void 0,this._$AD=t,this._$AM=i;}get parentNode(){return this._$AM.parentNode}get _$AU(){return this._$AM._$AU}u(t){const{el:{content:i},parts:s}=this._$AD,e=(t?.creationScope??r).importNode(i,true);C.currentNode=e;let h=C.nextNode(),o=0,n=0,l=s[0];for(;void 0!==l;){if(o===l.index){let i;2===l.type?i=new R(h,h.nextSibling,this,t):1===l.type?i=new l.ctor(h,l.name,l.strings,this,t):6===l.type&&(i=new z(h,this,t)),this._$AV.push(i),l=s[++n];}o!==l?.index&&(h=C.nextNode(),o++);}return C.currentNode=r,e}p(t){let i=0;for(const s of this._$AV) void 0!==s&&(void 0!==s.strings?(s._$AI(t,s,i),i+=s.strings.length-2):s._$AI(t[i])),i++;}}class R{get _$AU(){return this._$AM?._$AU??this._$Cv}constructor(t,i,s,e){this.type=2,this._$AH=E,this._$AN=void 0,this._$AA=t,this._$AB=i,this._$AM=s,this.options=e,this._$Cv=e?.isConnected??true;}get parentNode(){let t=this._$AA.parentNode;const i=this._$AM;return void 0!==i&&11===t?.nodeType&&(t=i.parentNode),t}get startNode(){return this._$AA}get endNode(){return this._$AB}_$AI(t,i=this){t=S(this,t,i),c(t)?t===E||null==t||""===t?(this._$AH!==E&&this._$AR(),this._$AH=E):t!==this._$AH&&t!==T&&this._(t):void 0!==t._$litType$?this.$(t):void 0!==t.nodeType?this.T(t):u(t)?this.k(t):this._(t);}O(t){return this._$AA.parentNode.insertBefore(t,this._$AB)}T(t){this._$AH!==t&&(this._$AR(),this._$AH=this.O(t));}_(t){this._$AH!==E&&c(this._$AH)?this._$AA.nextSibling.data=t:this.T(r.createTextNode(t)),this._$AH=t;}$(t){const{values:i,_$litType$:s}=t,e="number"==typeof s?this._$AC(t):(void 0===s.el&&(s.el=N.createElement(P(s.h,s.h[0]),this.options)),s);if(this._$AH?._$AD===e)this._$AH.p(i);else {const t=new M(e,this),s=t.u(this.options);t.p(i),this.T(s),this._$AH=t;}}_$AC(t){let i=A.get(t.strings);return void 0===i&&A.set(t.strings,i=new N(t)),i}k(t){a(this._$AH)||(this._$AH=[],this._$AR());const i=this._$AH;let s,e=0;for(const h of t)e===i.length?i.push(s=new R(this.O(l()),this.O(l()),this,this.options)):s=i[e],s._$AI(h),e++;e<i.length&&(this._$AR(s&&s._$AB.nextSibling,e),i.length=e);}_$AR(t=this._$AA.nextSibling,i){for(this._$AP?.(false,true,i);t&&t!==this._$AB;){const i=t.nextSibling;t.remove(),t=i;}}setConnected(t){ void 0===this._$AM&&(this._$Cv=t,this._$AP?.(t));}}class k{get tagName(){return this.element.tagName}get _$AU(){return this._$AM._$AU}constructor(t,i,s,e,h){this.type=1,this._$AH=E,this._$AN=void 0,this.element=t,this.name=i,this._$AM=e,this.options=h,s.length>2||""!==s[0]||""!==s[1]?(this._$AH=Array(s.length-1).fill(new String),this.strings=s):this._$AH=E;}_$AI(t,i=this,s,e){const h=this.strings;let o=false;if(void 0===h)t=S(this,t,i,0),o=!c(t)||t!==this._$AH&&t!==T,o&&(this._$AH=t);else {const e=t;let n,r;for(t=h[0],n=0;n<h.length-1;n++)r=S(this,e[s+n],i,n),r===T&&(r=this._$AH[n]),o||=!c(r)||r!==this._$AH[n],r===E?t=E:t!==E&&(t+=(r??"")+h[n+1]),this._$AH[n]=r;}o&&!e&&this.j(t);}j(t){t===E?this.element.removeAttribute(this.name):this.element.setAttribute(this.name,t??"");}}class H extends k{constructor(){super(...arguments),this.type=3;}j(t){this.element[this.name]=t===E?void 0:t;}}class I extends k{constructor(){super(...arguments),this.type=4;}j(t){this.element.toggleAttribute(this.name,!!t&&t!==E);}}class L extends k{constructor(t,i,s,e,h){super(t,i,s,e,h),this.type=5;}_$AI(t,i=this){if((t=S(this,t,i,0)??E)===T)return;const s=this._$AH,e=t===E&&s!==E||t.capture!==s.capture||t.once!==s.once||t.passive!==s.passive,h=t!==E&&(s===E||e);e&&this.element.removeEventListener(this.name,this,s),h&&this.element.addEventListener(this.name,this,t),this._$AH=t;}handleEvent(t){"function"==typeof this._$AH?this._$AH.call(this.options?.host??this.element,t):this._$AH.handleEvent(t);}}class z{constructor(t,i,s){this.element=t,this.type=6,this._$AN=void 0,this._$AM=i,this.options=s;}get _$AU(){return this._$AM._$AU}_$AI(t){S(this,t);}}const j=t$1.litHtmlPolyfillSupport;j?.(N,R),(t$1.litHtmlVersions??=[]).push("3.3.0");const B=(t,i,s)=>{const e=i;let h=e._$litPart$;if(void 0===h){const t=null;e._$litPart$=h=new R(i.insertBefore(l(),t),t,void 0,{});}return h._$AI(t),h};

  /**
   * @license
   * Copyright 2017 Google LLC
   * SPDX-License-Identifier: BSD-3-Clause
   */
  const t={CHILD:2},e$1=t=>(...e)=>({_$litDirective$:t,values:e});class i{constructor(t){}get _$AU(){return this._$AM._$AU}_$AT(t,e,i){this._$Ct=t,this._$AM=e,this._$Ci=i;}_$AS(t,e){return this.update(t,e)}update(t,e){return this.render(...e)}}

  /**
   * @license
   * Copyright 2017 Google LLC
   * SPDX-License-Identifier: BSD-3-Clause
   */class e extends i{constructor(i){if(super(i),this.it=E,i.type!==t.CHILD)throw Error(this.constructor.directiveName+"() can only be used in child bindings")}render(r){if(r===E||null==r)return this._t=void 0,this.it=r;if(r===T)return r;if("string"!=typeof r)throw Error(this.constructor.directiveName+"() called with a non-string value");if(r===this.it)return this._t;this.it=r;const s=[r];return s.raw=s,this._t={_$litType$:this.constructor.resultType,strings:s,values:[]}}}e.directiveName="unsafeHTML",e.resultType=1;const o=e$1(e);

  function closestHtml(child, attr) {
      let ptr = child;
      while (ptr && !ptr.getAttribute(attr)) {
          ptr = ptr.parentElement;
      }
      return ptr;
  }
  function confirmDialog({ editor, content: main, title, primaryLabel, secondaryLabel = 'Cancel', cbk, lsKey, }) {
      // Check if the user has already been asked
      if (localStorage.getItem(lsKey) === 'on') {
          cbk();
      }
      else {
          const content = document.createElement('div');
          editor.Modal.open({
              title,
              content,
          });
          let remember = 'off';
          B(x `<main>
        ${o(main)}
      </main><footer style="
        display: flex;
        justify-content: space-between;
        margin-top: 30px;
      ">
        <div>
          <label class="gjs-field gjs-field-checkbox" style="
            float: left;
            margin-right: 10px;
          ">
            <input type="checkbox" id="remember" @click=${({ target: rememberCheckbox }) => remember = rememberCheckbox === null || rememberCheckbox === void 0 ? void 0 : rememberCheckbox.value}>
            <i class="gjs-chk-icon"></i>
          </label>
          <label for="remember">Don't ask me again</label>
        </div>
        <div>
          <button
            class="gjs-btn-prim"
            @click=${() => editor.Modal.close()}
            style="
              margin-left: auto;
              background: transparent;
              margin-right: 10px;
            ">${secondaryLabel}</button>
          <button class="gjs-btn-prim" @click=${() => {
            cbk();
            localStorage.setItem(lsKey, remember);
            editor.Modal.close();
        }}>${primaryLabel}</button>
        </div>
      </footer>`, content);
      }
  }
  class SymbolsView extends Backbone.View {
      //initialize(model, { editor, options }) {
      // FIXME: why is editor in options?
      constructor(options) {
          super(options);
          this.options = options;
          this.lastPos = null;
          this.lastTarget = null;
          // listen to redraw UI
          if (!options.model)
              throw new Error('Could not create Symbol: model is required');
          options.model.on('add update remove', () => this.render());
          options.editor.on('component:selected', () => this.render());
          // listen to drag event in order to have access to the drop target
          options.editor.on('sorter:drag', event => {
              this.lastPos = event.pos;
              this.lastTarget = event.target;
          });
          // list wrapper
          this.el = document.createElement('div');
          this.el.classList.add('symbols__wrapper');
          document.querySelector(options.appendTo)
              .appendChild(this.el);
          // first render
          this.render();
      }
      render() {
          const symbols = this.model;
          const selected = this.options.editor.getSelected();
          B(x `
    <style>
      .symbols__symbol-selected {
        border: 1px solid ${this.options.highlightColor};
      }
      .symbols__symbol {
        position: relative;
      }
      .symbols__num {
        font-size: xx-small;
      }
      .symbols__empty {
        padding: 10px;
        text-align: center;
        width: 100%;
      }
      .symbols__remove {
        position: absolute;
        top: 0; right: 0;
        width: 20px;
        line-height: 1;
        cursor: pointer;
      }
    </style>
    <main class="symbols__list" @dragend=${(event) => this.onDrop(event)}>
      <div class="gjs-blocks-c">
      ${
        // keep the same structure as the layers panel
        symbols
            .map(s => x `
          <div
            class="gjs-block gjs-one-bg gjs-four-color-h symbols__symbol
              ${!!selected && s.get('instances').has(selected.cid) ? 'symbols__symbol-selected' : ''}
              fa ${s.attributes.icon}
            "
            title="" draggable="true"
            symbol-id=${s.cid}>
            <div title="Unlink all instances and delete Symbol" class="symbols__remove" @click=${(event) => this.onRemove(event)}>
              <svg viewBox="0 0 24 24"><path d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12 19 6.41z"></path></svg>
            </div>
            <div class="gjs-block-label">
              ${s.attributes.label}
              <div class="symbols__num">
                ${s.get('instances').size} instances
              </div>
            </div>
          </div>
         `)}
       ${symbols.length ? '' : x `<div class="symbols__empty">
        ${this.options.emptyText}
       </div>`}
       </div>
     </main>
   `, this.el);
          return this;
      }
      onDrop(event) {
          const symbolId = event.target.getAttribute('symbol-id');
          if (symbolId) {
              const symbol = this.options.editor.Symbols.get(symbolId);
              if (symbol) {
                  this.options.editor.runCommand('symbols:create', { symbol, pos: this.lastPos, target: this.lastTarget });
              }
              else {
                  console.error(`Could not create an instance of symbol ${symbolId}: symbol not found`);
              }
          }
      }
      onRemove({ target: deleteButton }) {
          // Warn the user
          confirmDialog({
              editor: this.options.editor,
              title: 'Delete Symbol',
              content: `
          <p>Are you sure you want to delete this symbol?</p>
          <p>Deleting this symbol <em>will not</em> delete its instances, just disconnects them. Confirm to proceed or cancel to maintain the current link.</p>
        `,
              primaryLabel: 'Delete',
              cbk: () => {
                  this.onRemoveConfirm(deleteButton);
              },
              lsKey: 'delete-symbol',
          });
      }
      onRemoveConfirm(target) {
          var _a;
          const symbolId = (_a = closestHtml((target), 'symbol-id')) === null || _a === void 0 ? void 0 : _a.getAttribute('symbol-id');
          if (symbolId) {
              this.options.editor.runCommand('symbols:remove', { symbolId });
          }
          else {
              console.error('not a symbol', symbolId);
          }
      }
  }

  var SymbolEvents;
  (function (SymbolEvents) {
      SymbolEvents["CREATE"] = "symbols:create";
      SymbolEvents["REMOVE"] = "symbols:remove";
      SymbolEvents["LINK"] = "symbols:link";
      SymbolEvents["UNLINK"] = "symbols:unlink";
      SymbolEvents["CREATE_INSTANCE"] = "symbols:create:instance";
      SymbolEvents["DELETE_INSTANCE"] = "symbols:delete:instance";
  })(SymbolEvents || (SymbolEvents = {}));

  const cmdAdd = 'symbols:add';
  const cmdRemove = 'symbols:remove';
  const cmdUnlink = 'symbols:unlink';
  const cmdCreate = 'symbols:create';
  // Same signature as a grapesjs plugin
  function initCommands (editor) {
      editor.Commands.add(cmdAdd, addSymbol);
      editor.Commands.add(cmdRemove, removeSymbol);
      editor.Commands.add(cmdUnlink, unlinkSymbolInstance);
      editor.Commands.add(cmdCreate, createSymbolInstance);
  }
  // Symbol management functions
  // These are exported for unit tests
  function displayError(editor, title, message) {
      const content = document.createElement('div');
      editor.Modal.open({
          title,
          content,
      });
      B(x `<main>
      <p>${o(message)}</p>
    </main><footer style="
      display: flex;
      justify-content: space-between;
      margin-top: 30px;
    ">
      <div></div>
      <button class="gjs-btn-prim" @click=${() => editor.Modal.close()}>Close</button>
    </footer>`, content);
  }
  /**
   * Create a new symbol
   * @param options.component - the component which will become the first instance of the symbol
   * @returns {Symbol}
   */
  function addSymbol(editor, _, { label, icon, component = editor.getSelected() }) {
      if (component && label && icon) {
          // add the symbol
          const s = editor.Symbols.add(createSymbol(editor, component, { label, icon }));
          setDirty(editor);
          // Notify plugins
          editor.trigger(SymbolEvents.CREATE, { symbol: s });
          // return the symbol to the caller
          return s;
      }
      else {
          console.error('Can not create the symbol: missing required param', { label, icon, component });
          throw new Error('Can not create the symbol: missing required param');
      }
  }
  /**
   * Delete a symbol
   * @param {symbolId: string} - object containing the symbolId
   */
  function removeSymbol(editor, _, { symbolId }) {
      if (symbolId) {
          if (editor.Symbols.has(symbolId)) {
              // remove the symbol
              const s = editor.Symbols.remove(symbolId);
              const instances = s.get('instances');
              // Unlink all instances
              s.unlinkAll();
              // notify the editor that a change occured
              setDirty(editor);
              // Notify the plugins
              instances.forEach((c) => editor.trigger(SymbolEvents.UNLINK, { symbol: s, component: c }));
              editor.trigger(SymbolEvents.REMOVE, { symbol: s });
              // return the symbol to the caller
              return s;
          }
          else {
              throw new Error('Could not remove symbol: symbol not found');
          }
      }
      else {
          throw new Error('Could not remove symbol: missing param symbolId');
      }
  }
  function unlinkSymbolInstance(editor, _, { component }) {
      if (component) {
          const s = editor.Symbols.get(getSymbolId(component));
          if (s) {
              s.unlink(component);
              // notify the editor that a change occured
              setDirty(editor);
              // Notify the plugins
              editor.trigger(SymbolEvents.UNLINK, { symbol: s, component });
          }
          else {
              console.warn('Can not unlink component', component, 'Symbol not found');
          }
      }
      else {
          throw new Error('Can not unlink the component: missing param component');
      }
  }
  /**
   * @param {{index, indexEl, method}} pos Where to insert the component, as [defined by the Sorter](https://github.com/artf/grapesjs/blob/0842df7c2423300f772e9e6cdc88c6ae8141c732/src/utils/Sorter.js#L871)
   */
  function createSymbolInstance(editor, _, { symbol, pos, target }) {
      pos = pos || {};
      if (symbol && pos && target) {
          const parentId = target ? target.getAttribute('id') : undefined;
          if (!parentId)
              throw new Error('Can not create the symbol: missing param id');
          const parent = target ? editor.Components.allById()[parentId] : undefined;
          // Check that it is a valid parent
          if (parent) {
              if (!allowDrop({ target: symbol.get('model'), parent })) {
                  // Cancel and notify the user
                  displayError(editor, 'Error: can not create the symbol', '<p>One of the parent is in the symbol.</p><p>Please remove the parent from the symbol and try again.</p>');
                  return null;
              }
              else {
                  // create the new component
                  const [c] = parent ? parent.append([symbol.createInstance()], { at: pos.index }) : [];
                  // Select the new component
                  // Break unit tests? editor.select(c, { scroll: true })
                  // Notify plugins
                  editor.trigger(SymbolEvents.LINK, { symbol, component: c });
                  editor.trigger(SymbolEvents.CREATE_INSTANCE, { symbol, component: c });
                  return c;
              }
          }
          else {
              console.error('Can not create the symbol: parent not found');
              return null;
          }
      }
      else {
          throw new Error('Can not create the symbol: missing param symbol or pos or target');
      }
  }

  // Same signature as a grapesjs plugin
  function initTraits (editor, options) {
      function setSync(el, component, sync) {
          component.set(SYMBOL_SYNC_ATTRIBUTE, sync);
          updateUi(el, component);
      }
      function unlink(component) {
          confirmDialog({
              editor,
              title: 'Unlink from symbol',
              content: `
        <p>Are you sure you want to unlink this component from the symbol? This is a <em>definitive action<em></p>
        <p>Unlinking this component <em>will not</em> delete the symbol, just disconnects it. Confirm to proceed or cancel to maintain the current link.</p>
      `,
              primaryLabel: 'Unlink',
              cbk: () => {
                  editor.runCommand(cmdUnlink, { component });
              },
              lsKey: 'unlink-symbol',
          });
      }
      function updateUi(el, component) {
          var _a;
          const symbolId = component && getSymbolId(component);
          if (symbolId) {
              const sync = component.get(SYMBOL_SYNC_ATTRIBUTE) !== false;
              el.style.display = 'initial';
              B(x `<fieldset class="gjs-trt-trait__wrp gjs-trt-trait__wrp-title" style="
        border-color: ${options.primaryColor};
        padding: 10px;
      ">
        <legend class="fa fa-ban on fa-diamond"> <em>${(_a = editor.Symbols.get(symbolId)) === null || _a === void 0 ? void 0 : _a.get('label')}</em></legend>
        <div class="gjs-field">
          <button
            class="gjs-btn-prim gjs-btn--full"
            style="
              margin: 10px 0;
              border: 1px solid ${options.primaryColor};
            "
            @click=${() => unlink(component)}>Unlink</button>
          <div class="gjs-radio-items">
            <label class="gjs-radio-item">
              <input type="radio" name="sync" value="on" @click=${() => setSync(el, component, true)} ?checked=${sync}/>
              <span class="gjs-radio-item-label" style="color: ${sync ? options.highlightColor : options.primaryColor}">ON</span>
            </label>
            <label class="gjs-radio-item">
              <input type="radio" name="sync" value="off" @click=${() => setSync(el, component, false)} ?checked=${!sync}/>
              <span class="gjs-radio-item-label" style="color: ${sync ? options.primaryColor : options.highlightColor}">OFF</span>
            </label>
        </div>
      </fieldset>`, el);
          }
          else {
              el.style.display = 'none';
          }
      }
      // Create a new trait type
      editor.TraitManager.addType('symbol-trait', {
          noLabel: true,
          createInput() {
              // Create a new element container and add some content
              const el = document.createElement('div');
              // update the UI when a new symbol is selected or created/deleted etc
              Object.values(SymbolEvents).forEach(event => editor.on(event, () => updateUi(el)));
              // Initial ui update
              updateUi(el);
              // this will be the element passed to onEvent and onUpdate
              return el;
          },
          // Update the component based on UI changes
          // `elInput` is the result HTMLElement you get from `createInput`
          //onEvent({ elInput, component, event }) {
          //},
          // Update UI on the component change
          onUpdate({ elInput, component }) {
              updateUi(elInput, component);
          },
      });
      // Add trait to symbols when the user selects one
      editor.on('component:selected', component => {
          const symbolId = component && getSymbolId(component);
          if (symbolId) {
              component.addTrait({
                  type: 'symbol-trait',
                  name: 'Symbol',
              });
          }
      });
      //// Add the new trait to all component types
      //editor.DomComponents.getTypes().forEach(type => {
      //  editor.DomComponents.addType(type.id, {
      //    //isComponent: el => isComponent(el),
      //    model: {
      //      defaults: {
      //        traits: [
      //          // Keep the type original traits
      //          ...editor.DomComponents.getType(type.id).model.prototype.defaults.traits,
      //          // Add the new trait
      //          {
      //            type: 'symbol-trait',
      //            name: 'Symbol',
      //          }
      //        ]
      //      }
      //    }
      //  })
      //})
  }

  var index = (editor, opts = {}) => {
      const options = { ...{
              appendTo: '#symbols',
              emptyText: 'No symbol yet.',
              primaryColor: '#b9a5a6',
              secondaryColor: '#463a3c',
              highlightColor: '#d97aa6',
          }, ...opts };
      // store the symbols data with the site
      Storage(editor);
      // keep track of symbols and changes
      editor.Symbols = new Symbols([], { options, editor });
      editor.on('load', () => {
          // Display symbols
          new SymbolsView({ ...options, editor, model: editor.Symbols, });
          // Commands to create/delete symbols
          initCommands(editor);
          // Add traits like a plugin
          initTraits(editor, options);
      });
  };

  return index;

})(Backbone);
